"======================================================================
|
|   Smalltalk parser definitions.
|
|   $Revision: 1.6 $
|   $Date: 1995/09/17 10:48:51 $
|   $Author: sbb $
|
 ======================================================================"


"======================================================================
|
| Copyright (C) 1994, 1995 Free Software Foundation, Inc.
| Written by Steve Byrne.
|
| This file is part of GNU Smalltalk.
|
| GNU Smalltalk is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 1, or (at your option) any later version.
| 
| GNU Smalltalk is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| GNU Smalltalk; see the file COPYING.  If not, write to the Free Software
| Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
|
 ======================================================================"


"
|     Change Log
| ============================================================================
| Author       Date       Change 
|
"

Object subclass: #Compiler
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Compiler'
!

Compiler subclass: #ParseNode
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Compiler'
!

Compiler subclass: #Parser
       instanceVariableNames: 'stream curCategory curClass'
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Compiler'
!


ParseNode comment:
'I am the root of compiler''s node hierarchy.  All parse tree nodes are
descendants of me.'!

FileStream fileIn: 'tok.st'!
FileStream fileIn: 'RParseNodes.st'!
FileStream fileIn: 'STCompiler.st'!



" need internal state:

stream
current class & category
"

!Object methodsFor: 'fun hacks'!

recursiveInspect
    | instVars |
    self inspect.
    instVars _ self class allInstVarNames.
    1 to: instVars size do:
    	[ :i | stdout nextPutAll: '  ';
		   nextPutAll: (instVars at: i);
		   nextPutAll: ': '.
	       (self instVarAt: i)  recursiveInspect ]
! !


!Parser class methodsFor: 'instance creation'!

on: aFileName
    ^self new init: aFileName
! !


!Parser methodsFor: 'private'!

init: aFileName
    stream _ Tok on: aFileName
! !

!Parser methodsFor: 'tidyness'!

close				"probably there is a better name for this"
    stream close
!!

"
| p |
    p _ Parser on: 'simple.st'.
    p parseSmalltalk
!
"

"
| p |
    p _ Parser on: 'simple2.st'.
    p parseSmalltalk
!
"

"
| p |
    p _ Parser on: 'ParseNodes.st'.
    p parseSmalltalk
!
"
 
"
| p |
 #(
  'builtins.st' 
  'Object.st'
  'Message.st'
  'Magnitude.st'
  'Character.st'
  'Date.st'
  'Time.st'
  'Number.st'
  'Float.st'
  'Integer.st'
  'LookupKey.st'
  'Association.st'
  'Link.st'
  'Process.st'
  'Collection.st'
  'SequenceableCollection.st'
  'LinkedList.st'
  'Semaphore.st'
  'ArrayedCollection.st'
  'Array.st'
  'String.st'
  'Symbol.st'
  'ByteArray.st'
  'CompiledMethod.st'
  'Interval.st'
  'OrderedCollection.st'
  'SortedCollection.st'
  'Bag.st'
  'MappedCollection.st'
  'Set.st'
  'Dictionary.st'
  'IdentityDictionary.st'
  'SystemDictionary.st'
  'Stream.st'
  'PositionableStream.st'
  'ReadStream.st'
  'WriteStream.st'
  'ReadWriteStream.st'
  'FileStream.st'
  'TokenStream.st'
  'Random.st'
  'UndefinedObject.st'
  'Boolean.st'
  'False.st'
  'True.st'
  'ProcessorScheduler.st'
  'Delay.st'
  'SharedQueue.st'
  'Behavior.st'
  'ClassDescription.st'
  'Class.st'
  'Metaclass.st'
  'MethodContext.st'
  'BlockContext.st'
  'Memory.st'
  'WordMemory.st'
  'ByteMemory.st'
  'MethodInfo.st'
  'FileSegment.st'
  'SymLink.st'
  'initialize.st'
  'CFuncs.st'
  'CType.st'
  'CObject.st'
  'CStruct.st'
  'Autoload.st'
  'Fraction.st'
  'UnixStream.st'
  'IOCtl.st'
  'changes.st' 
  'Browser.st'

) do:
   [ :file | p _ Parser on: file.
             p parseSmalltalk.
             p close ].

!
"

!Parser methodsFor: 'parsing'!

parseSmalltalk
    | t stmts body |
    [ stream resetRecording.
      t _ stream peek.
      stream atEnd ] whileFalse:
	[ 
	  t isNil ifTrue: [ ^self ].
	  (t isMemberOf: BangTok)
	      ifTrue: [ stream next. "gobble bang"
			self parseMethodDefinitionList ]
	      ifFalse: [ body _ self parseMethodBody.
			 "### build a method body and then execute"
			 '

### inspecting experssion' printNl.
			 body recursiveInspect.
			 stream next. "gobble terminating bang"
			 "stmts execute" ]
	      ].
!

" Need some cool exception mechanism, like a return from a block or something"

parseMethodDefinitionList
    "Called after first !, expecting a methodListHeader (
        <name> methodsFor: <category string> !), then a set of bang
     terminated method definitions, followed by a bang"
    | t nodes compiler |
    self parseMethodListHeader.
    [ t _ stream peek.
      stdout nextPutAll: 'found '; print: t; nl.
      t isMemberOf: BangTok ] whileFalse:
	  [
	   nodes _ self parseMethodDefinition.
	   stream recording do: [ :elem | stdout nextPut: elem ].
	   "The object boundaries are not clear to me.  It seems right that 
	    the compiler is an object, but is it created with just a set of 
	    nodes which it can then operate on in various interesting ways, or
	    is the main method compileNodes: and it's just created via #new?"
	   compiler _ STCompiler onNodes: nodes.
	   compiler compile.
	   "nodes compile."
	   stream next.		"gobble method terminating bang"
	   stream resetRecording.
	   ].
    stream next.		"gobble definition-list terminating bang"
!

parseMethodListHeader
    | nodes |
    " Parses the stuff between the bangs in 
        ! <class expression> methodsFor: <category string> !
     "
    nodes _ self parseSimpleExpression.	"???Justification for 'simple' here"
    nodes isNil
	ifTrue: [ ^self parseError: 'failed to parse MethodListHeader' ].
    nodes execute.		"may not be correct"
    stream next.		"gobble terminating bang"
!

parseMethodDefinition
    " parse a method definition, from selector, to terminating bang "
    | selector body |

    selector _ self parseSelector.
    body _ self parseMethodBody.
    ^MethodDefinitionNode new: selector body: body
!
    
parseSelector
    | t sel | 
    t _ stream peek.
    (t isMemberOf: IdentifierTok) ifTrue: 
	[ ^Selector selector: stream next value asSymbol args: #()].
    (t isMemberOf: KeywordTok) ifTrue: [ ^self parseKeywordSelector ].
    (self isBinaryOperator: t) ifTrue: 
	[ sel _ stream next.
	  t _ stream next.
	  (t isMemberOf: IdentifierTok) ifFalse: 
	      [ ^self parseError: 'expected identifier to follow binary op' ].
	  ^Selector selector: sel args: (Array with: t) ].
    ^self parseError: 'invalid method selector'
!

parseKeywordSelector
    | t selector args | 
    selector _ ''.
    args _ OrderedCollection new: 0.
    [ t _ stream peek.
      t isMemberOf: KeywordTok ] whileTrue: 
	  [
	   selector _ selector , stream next value.
	   t _ stream next.
	   (t isMemberOf: IdentifierTok) ifFalse:
	       [ ^self parseError: 'expected identifer after keyword' ].
	   args add: t.
	   ].
    ^Selector selector: selector args: args
!

parseMethodBody
    | t temporaries primitiveIndex statements | 
    t _ stream peek.
    (t isMemberOf: VerticalBarTok)
	ifTrue: [ stream next.	"gobble vertical bar"
		  temporaries _ self parseTemporaries.
		  t _ stream peek ]
	ifFalse: [ temporaries _ #() ].
    (t isMemberOf: PrimitiveStartTok)
	ifTrue: [ stream next.	"gobble primitive start"
		  primitiveIndex _ self parsePrimitive.
		  ].
    
    statements _ self parseStatements.
    ^MethodBodyNode temps: temporaries primIndex: primitiveIndex
		    stmts: statements
!


parseTemporaries
    "Parses 
         | << <name> <name> ... | >> 
     and returns the list of names"
    | t temps |
    temps _ OrderedCollection new: 0.
    [ t _ stream peek.
      t isMemberOf: VerticalBarTok ] whileFalse: 
	  [ temps add: self parseIdentifier ].
    stream next.		"gobble vertical bar"
    ^temps
!

parseIdentifier
    | id |
    id _ stream next.
    (id isMemberOf: IdentifierTok)
	ifTrue: [ ^id ]
	ifFalse: [ ^self parseError: 'expected identifier' ]
!

parsePrimitive
    | int t |
    int _ stream next.
    ((int isMemberOf: LiteralTok) 
	 and: [ int value isMemberOf: Integer ] )
	ifFalse: [ ^self parseError: 'primitive: must be followed by integer literal' ].
    t _ stream next.
    ((t isMemberOf: BinopTok)
	and: [ t value = '>' ])
	ifFalse: [ ^self parseError: 'invalid terminator for primitive:, expecting ''>''' ].
    "### may need to fix int to be some internal parse node type"
    ^int
!



"parseStatementList: endingToken
    | t stmts expr |
    stmts _ OrderedCollection new.
    [ t _ stream peek.
      t isMemberOf: endingToken ] whileFalse:
	  [ whileFalse: 
        [ 
	  (t isMemberOf: UpArrowTok)
	      ifTrue: [ expr _ self parseReturnExpr.
			""### do some kind of return type thing"" ]
	      ifFalse: [ expr _ self parseExpr ].
	  
	  stmts add: expr.	""??? is there a need for a statement type node?""
	  t _ stream peek.
	  (t isMemberOf: DotTok) ifTrue: 
	      [ stream next.	""gobble terminating dot"" ].
	  ]
      ].
    ^stmts
!
"

parsePrimary
    | t expr |
    t _ stream peek.
    (t isMemberOf: IdentifierTok) ifTrue: [ ^IdentifierNode id: stream next value ].
    (t isMemberOf: LiteralTok) ifTrue: [ ^LiteralNode value: stream next value].
    " ### should these be identifiers? "
    (self isBinaryOperator: t) 
	ifTrue: [ ^IdentifierNode id: stream next value asSymbol ].
    (t isMemberOf: SharpTok) ifTrue: [ stream next.
				       ^self parseSharpConstant ].
    (t isMemberOf: OpenBracketTok) ifTrue: 
	[ stream next.		"gobble"
	  ^self parseBlock ].
    (t isMemberOf: OpenParenTok) ifTrue: 
	[ stream next.		"gobble gobble"
	  expr _ self parseExpression.
	  expr isNil 
	      ifTrue: [ ^self parseError: 'Missing parenthesized expression' ].
	  t _ stream next.
	  (t isMemberOf: CloseParenTok)
	      ifTrue: [ ^ExpressionNode expression: expr ]
	      ifFalse: [ ^self parseError: 'Expecting close paren' ] ].
    ((t isMemberOf: CloseBracketTok) or: [ (t isMemberOf: BangTok) ])
	ifTrue: [ ^nil ].
	 
    ^self parseError: 'Unrecognized expression'
!


parseSharpConstant
    " Called at
         # << id or ( lit lit lit ) >> "
    | t lit |
    t _ stream next.
    (self isSymbolId: t) ifTrue: [ ^SymbolNode id: t value asSymbol ].
    (t isMemberOf: OpenParenTok) ifTrue: [ lit _ self parseArrayLit.
					   ^lit ].
    ^self parseError: 'Unrecognized literal'
!

parseArrayLit
    "Called at
         ( << id or binop or number or char or string or ( array ) or empty ) >>"
    | t array |
    array _ OrderedCollection new.
    [ t _ stream next.
      t isMemberOf: CloseParenTok ] whileFalse:
	  [ (t isMemberOf: LiteralTok)
		ifTrue: [ array add: t value ]
		ifFalse: 
		    [ (self isSymbolId: t) 
			  ifTrue: [ array add: (SymbolNode id: t value asSymbol) ]
			  ifFalse: 
			      [ (t isMemberOf: OpenParenTok)
				    ifTrue: [ array add: self parseArrayLit ] 
				    ifFalse: [ ^self parseError: 'Bad array literal' ]
				]
			  ].
	    ].
    ^ArrayConstNode with: array
!
    

parseBlock
    " Called at
     [ << block_identifiers ... | statements ] >> 
     "
    | t identifiers statements |
    t _ stream peek.
    (t isMemberOf: ColonTok)
	ifTrue: [ identifiers _ self parseBlockIdentifiers.
		  t _ stream peek ].
    statements _ self parseStatements.
    t _ stream next.
    (t isMemberOf: CloseBracketTok)
	ifFalse: [ ^self parseError: 'bad block syntax' ].
    ^BlockNode parameters: identifiers statements: statements
!

parseBlockIdentifiers
    "Called at
      [ << :blockparam :blockparam ... | >> "
    | t identifiers |
    identifiers _ OrderedCollection new: 0.
    [ t _ stream next.
      t isMemberOf: VerticalBarTok ] whileFalse:
	  [ (t isMemberOf: ColonTok)
		ifFalse: [ ^self parseError: 'Bad block param syntax' ].
	    t _ stream next.
	    (t isMemberOf: IdentifierTok)
		ifFalse: [ ^self parseError: 'Bad block param syntax' ].
	    identifiers add: (IdentifierNode id: t value)
	    ].
    ^identifiers
!
    
parseStatements
    | expression t list done |
    list _ OrderedCollection new: 0.
    done _ false.
    [ done ] whileFalse:
	[ t _ stream peek.
	  (t isMemberOf: UpArrowTok)
	      ifTrue: [ stream next. "skip ^"
			list add: self parseReturnNode.
			done _ true ]
	      ifFalse: [ expression _ self parseExpression.
			 expression isNil
			     ifTrue: [ done _ true ]
			     ifFalse: [ list add: expression.
					t _ stream peek.
					(t isMemberOf: DotTok)
					    ifTrue: [ stream next "gobble it" ]
					    ifFalse: [ done _ true ]
					    ]
			     ]
	      ].
    ^list
!

parseReturnNode
    | expression |
    expression _ self parseExpression.
    ((stream peek) isMemberOf: DotTok)
	ifTrue: [ stream next ].
    ^ReturnNode expression: expression
!


parseExpression
    | expr assigns t |
    assigns _ OrderedCollection new: 0.
    [ expr _ self parseSimpleExpression.
      t _ stream peek.
      t isMemberOf: AssignTok ] whileTrue:
	  [ stream next.
	    (expr isMemberOf: IdentifierNode)
		ifFalse: [ ^self parseError: 'Invalid assignment variable' ].
	    assigns add: expr ].
    expr isNil
	ifTrue: [ ^nil ]
	ifFalse: [ ^ExpressionNode assign: assigns expression: expr ]
!

parseSimpleExpression
    ^self parseCascadedExpr
!

parseCascadedExpr
    | message expression |
    expression _ self parsePrimary.
    "This feels like it should be recursive, but it's not currently"
    [ message _ self parseKeywordMessage.
      message notNil ]
	whileTrue: [ expression _ MessageNode receiver: expression
					   message: message ].
    ^self parseCascadedMessage: expression.
!

parseCascadedMessage: expression
    | t message cascadeList | 
    cascadeList _ OrderedCollection new: 0.
    
    "This probably isn't needed"
    "(expression isMemberOf: MessageNode)
	ifFalse: [ expression inspect.
		   ^self parseError: 'Selector required' ]."
		  
    [ t _ stream peek.
      (t isMemberOf: SemiTok) ]
	whileTrue: 
	    [ stream next.	"gobble semicolon"
	      message _ self parseKeywordMessage.
	      message isNil 
		  ifTrue: [ ^self parseError: 'Bogus cascaded expression' ].
		  
	      cascadeList addLast: message.
	].

    cascadeList size == 0
	ifTrue: [ ^expression ]
	ifFalse: [ ^CascadeNode expression: expression cascade: cascadeList ]
!
    

"parseKeywordExpr
    | binop |
    binop _ self parseBinopExpr.
    ^self parseKeywordMessage: binop
!

parseKeywordMessage: binop
    | t selector exprs |
    t _ stream peek.
    (t isMemberOf: KeywordTok)
      ifTrue: [ selector _ stream next value asSymbol.
		exprs _ OrderedCollection with: self parseBinopExpr.
		[ t _ stream peek. 
		  t isMemberOf: KeywordTok ] whileTrue:
		      [ selector _ selector , stream next value.
			exprs add: self parseBinopExpr ].
		^KeywordExpr receiver: binop
			     selector: selector
			     expressions: exprs 
		   ]
      ifFalse: [ ^binop ]
!
      
parseBinopExpr
    | unop |
    unop _ self parseUnopExpr.
    ^self parseBinaryMessage: unop.
!

parseBinaryMessage: unaryExpr
    | t selector operand |
    t _ stream peek.
    (self isBinaryOperator: t)
	ifTrue: [ selector _ stream next.
		  operand _ self parseUnopExpr.
		  ^self parseBinaryMessage: 
		      (BinaryExprNode receiver: unaryExpr
				      selector: selector
				      expression: operand ) ]
	ifFalse: [ ^unop ]
!

parseUnopExpr
    | primary |
    primary _ self parsePrimary.
    ^self parseUnaryMessage: primary
!

parseUnaryMessage: expr
    | t selector |
    t _ stream peek.
    (t isMemberOf: IdentifierTok)
	ifTrue: [ selector _ stream next.
		  ^self parseUnaryMessage: (UnaryExprNode expr: expr 
							  selector: selector) ]
	ifFalse: [ ^expr ]
!
"

parseKeywordMessage
    | t selector exprs |
    t _ stream peek.
    (t isMemberOf: KeywordTok)
	ifTrue: [ selector _ ''.
		  exprs _ OrderedCollection new.
		  [ t _ stream peek. 
		    t isMemberOf: KeywordTok ] whileTrue:
			[ selector _ selector , stream next value.
			  exprs add: self parseBinopExpr ].
		  ^KeywordNode selector: selector asSymbol
			       expressions: exprs ]
	ifFalse: [ ^self parseBinaryMessage ]
!    

parseBinopExpr
    | t receiver message |
    receiver _ self parseUnaryExpr.
    ^self parseBinaryMessage: receiver
!

parseBinaryMessage: expression
    | message |
    message _ self parseBinaryMessage.
    message notNil
	ifTrue: [ ^self parseBinaryMessage:
		      (MessageNode receiver: expression
				   message: message) ]
	ifFalse: [ ^expression ]
!

parseBinaryMessage
    | t operand |
    t _ stream peek.
    (self isBinaryOperator: t)
	ifTrue: [ stream next.
		  operand _ self parseUnaryExpr.
		  ^BinaryNode selector: t
			      expression: operand ]
	ifFalse: [ ^self parseUnaryMessage ]
!


parseUnaryExpr
    | receiver |
    receiver _ self parsePrimary.
    ^self parseUnaryMessage: receiver
!

parseUnaryMessage: expression
    | message |
    message _ self parseUnaryMessage.
    message notNil
	ifTrue: [ ^self parseUnaryMessage:
		      (MessageNode receiver: expression
				   message: message) ]
	ifFalse: [ ^expression ]
!

parseUnaryMessage
    | t | 
    t _ stream peek.
    (t isMemberOf: IdentifierTok)
	ifTrue: [ ^UnaryNode selector: stream next ]
	ifFalse: [ ^nil ]
!


"!!! Private"

parseError: str
    stdout nextPutAll: '
Parse Error encountered
=======================

Scan so far was: 
'.
    stream recording do:
	[ :elem | stdout nextPut: elem ].
    stdout nl.
    ^self error: str
!

isSymbolId: t
    ^(t isMemberOf: IdentifierTok)
	or: [ (t isMemberOf: KeywordTok) 
		  or: [ (t isMemberOf: SymbolKeywordTok)
			    or: [ self isBinaryOperator: t ] ] ]
!


isBinaryOperator: token
    ^(token isMemberOf: BinopTok) or: [ (token isMemberOf: VerticalBarTok) ]
! !




"
GenClass generate: stdout parent: #ParseNode 
	 names:
	     #(
	       BinOp
		   UnaryOp
		   Statement
		   AssignStatement
		   MethodDefinition
		   ClassDefinition
		   StatementList
		   KeywordExpr
		   BinaryExpr
		   UnaryExpr
		   KeywordList
		   Variable
		   IntConst
		   FloatConst
		   SymbolConst
                   Literal
		   Identifier
		   StringConst
		   ArrayConst
		   Block
		   ArrayElt
		   VariableList
		   CascadedMessage
		   )
	 suffix: #Node
!
"
