"======================================================================
|
|   (Parts of ) A Smalltalk parser.
|
|   $Revision: 1.5 $
|   $Date: 1995/09/12 04:33:59 $
|   $Author: sbb $
|
 ======================================================================"


"======================================================================
|
| Copyright (C) 1994, 1995 Free Software Foundation, Inc.
| Written by Steve Byrne.
|
| This file is part of GNU Smalltalk.
|
| GNU Smalltalk is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 1, or (at your option) any later version.
| 
| GNU Smalltalk is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| GNU Smalltalk; see the file COPYING.  If not, write to the Free Software
| Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
|
 ======================================================================"


"
|     Change Log
| ============================================================================
| Author       Date       Change 
| sbb	      4 Sep 95	  Switched to being externally accessible instead of
|			  supporting internal access only.
|
"

ParseNode subclass: #Selector
	  instanceVariableNames: 'selector args'
	  classVariableNames: ''
	  poolDictionaries: ''
	  category: ''
!

!Selector class methodsFor: 'instance creation'!

selector: sel args: args
    ^self new init: sel args: args
! !


!Selector methodsFor: 'accessing'!

selector
    ^selector
!

args
    ^args
! !



!Selector methodsFor: 'private'!

init: aSelector args: argList
    selector _ aSelector.
    args _ argList
! !

ParseNode subclass: #BinOpNode
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

ParseNode subclass: #UnaryOpNode
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

ParseNode subclass: #StatementNode
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

ParseNode subclass: #AssignStatementNode
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

ParseNode subclass: #MethodDefinitionNode
    instanceVariableNames: 'selector body'
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

!MethodDefinitionNode class methodsFor: 'instance creation'!

new: selector body: methodBody
    ^self new init: selector body: methodBody
! !


!MethodDefinitionNode methodsFor: 'accessing'!

selector
    ^selector
!

body
    ^body
! !


!MethodDefinitionNode methodsFor: 'compilation'!

compile
    | bodyBytecodes | 
    stdout nl; nl; nextPutAll: '[compiling]'; nl.
    self recursiveInspect.
    stdout nl; nl.
! !


!MethodDefinitionNode methodsFor: 'private'!

init: aSelector body: methodBody
    selector _ aSelector.
    body _ methodBody
! !



ParseNode subclass: #MethodBodyNode
    instanceVariableNames: 'temporaries primitive statements'
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

!MethodBodyNode class methodsFor: 'instance creation'!

temps: temps primIndex: pi stmts: stmts
    ^self new init: temps primIndex: pi stmts: stmts
! !


!MethodBodyNode methodsFor: 'accessing'!

temporaries
    ^temporaries
!

primitive
    ^primitive
!

statements
    ^statements
! !


!MethodBodyNode methodsFor: 'private'!

init: temps primIndex: pi stmts: stmts
    temporaries _ temps.
    primitive _ pi.
    statements _ stmts
! !


ParseNode subclass: #SymbolNode
    instanceVariableNames: 'identifier'
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

!SymbolNode class methodsFor: 'instance creation'!

id: id
    ^self new init: id
! !

!SymbolNode methodsFor: 'private'!

init: id
    identifier _ id
! !

ParseNode subclass: #CascadeNode
    instanceVariableNames: 'receiver message'
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

!CascadeNode class methodsFor: 'instance creation'!

expression: expr cascade: msg
    ^self new init: expr cascade: msg
! !


!CascadeNode methodsFor: 'private'!

init: expr cascade: msg
    receiver _ expr.
    message _ msg
! !


ParseNode subclass: #MessageNode
    instanceVariableNames: 'receiver message'
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

!MessageNode class methodsFor: 'instance creation'!

receiver: rec message: mess
    ^self new init: rec message: mess
! !


!MessageNode methodsFor: 'testing'!

execute
    "### fix"
    'invoked MessageNode execute' printNl.
    self recursiveInspect.
    ^self
! !


!MessageNode methodsFor: 'private'!

init: rec message: mess
    receiver _ rec.
    message _ mess
! !


ParseNode subclass: #ClassDefinitionNode
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

ParseNode subclass: #StatementListNode
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

ParseNode subclass: #ReturnNode
    instanceVariableNames: 'expression'
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

!ReturnNode class methodsFor: 'instance creation'!

expression: expr
    ^self new init: expr
! !


!ReturnNode methodsFor: 'accessing'!

expression
    ^expression
! !



!ReturnNode methodsFor: 'private'!

init: expr
    expression _ expr.
! !


ParseNode subclass: #ExpressionNode
    instanceVariableNames: 'assigns expression'
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

!ExpressionNode class methodsFor: 'instance creation'!

expression: expr
    ^self new init: expr
!

assign: assignsList expression: expr
    ^self new init: assignsList expression: expr
! !



!ExpressionNode methodsFor: 'accessing'!

assigns
    ^assigns
!

expression
    ^expression
! !



!ExpressionNode methodsFor: 'private'!

init: assignsList expression: expr
    assigns _ assignsList.
    expression _ expr
!

init: expr
    expression _ expr		"assign list is empty"
! !


ParseNode subclass: #KeywordNode
    instanceVariableNames: 'selector expressions'
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

!KeywordNode class methodsFor: 'instance creation'!

selector: selector expressions: exprs
    ^self new init: selector expressions: exprs
! !

!KeywordNode methodsFor: 'private'!

init: aSelector expressions: exprs
    selector _ aSelector.
    expressions _ exprs
! !


ParseNode subclass: #BinaryNode
    instanceVariableNames: 'selector expression'
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

!BinaryNode class methodsFor: 'instance creation'!

selector: sel expression: expr
    ^self new init: sel expression: expr
! !

!BinaryNode methodsFor: 'private'!

init: sel expression: expr
    sel _ selector.
    expression _ expr
! !

ParseNode subclass: #UnaryNode
    instanceVariableNames: 'selector'
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

!UnaryNode class methodsFor: 'instance creation'!

selector: sel
    ^self new init: sel
! !

!UnaryNode methodsFor: '[private'!

init: sel
    selector _ sel
! !

ParseNode subclass: #KeywordListNode
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

ParseNode subclass: #VariableNode
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

ParseNode subclass: #IntConstNode
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

ParseNode subclass: #FloatConstNode
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

ParseNode subclass: #SymbolConstNode
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!


ParseNode subclass: #IdentifierNode
    instanceVariableNames: 'id'
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

!IdentifierNode class methodsFor: 'instance creation'!

id: identifier
    ^self new init: identifier
! !


!IdentifierNode methodsFor: 'accessing'!

id
    ^id
! !



!IdentifierNode methodsFor: 'private'!

init: identifier
    id _ identifier
! !



ParseNode subclass: #StringConstNode
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

ParseNode subclass: #ArrayConstNode
    instanceVariableNames: 'array'
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

!ArrayConstNode class methodsFor: 'instance creation'!

with: anArray
    ^self new init: anArray
! !

!ArrayConstNode methodsFor: 'private'!

init: anArray
    array _ anArray
! !


ParseNode subclass: #LiteralNode
    instanceVariableNames: 'value'
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

!LiteralNode class methodsFor: 'instance creation'!

value: value
    ^self new init: value
! !

!LiteralNode methodsFor: 'private'!

init: val
    value _ val
! !


ParseNode subclass: #BlockNode
    instanceVariableNames: 'parameters statements'
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

!BlockNode class methodsFor: 'instance creation'!

parameters: params statements: stmts
    ^self new init: params statements: stmts
! !

!BlockNode methodsFor: 'private'!

init: params statements: stmts
    parameters _ params.
    statements _ stmts
! !


ParseNode subclass: #ArrayEltNode
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

ParseNode subclass: #VariableListNode
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

ParseNode subclass: #CascadedMessageNode
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''
    category: ''
!

