"======================================================================
|
|   Smalltalk Blox based class browser example
|
|   $Revision: 1.3 $
|   $Date: 1995/07/18 06:26:35 $
|   $Author: sbb $
|
 ======================================================================"

 

"======================================================================
|
| Copyright (C) 1994, 1995 Free Software Foundation, Inc.
| Written by Steve Byrne.
|
| This file is part of GNU Smalltalk.
|
| GNU Smalltalk is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 1, or (at your option) any later version.
| 
| GNU Smalltalk is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| GNU Smalltalk; see the file COPYING.  If not, write to the Free Software
| Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
|
 ======================================================================"

"
|     Change Log
| ============================================================================
| Author       Date       Change 
| sbb	     16 Jul 95	  More updates to contextInspector>>init: from bdiller.
|
| sbb	     16 Jul 95	  Updated with bdiller's changes/fixes.  Removed
|			  #changed:/#update: usage.
|
| sbb	      9 Jul 94	  Created about a week ago.
|
"

FileStream require: #Blox -> 'Blox.st'!


"----------------------------------------------------------------------
from here down are hacks to get the browser back on the air.  
These are not part of the normal code.
"

Smalltalk at: #HandleErrorsWithGui put: false!


"
----------------------------------------------------------------------
"


Gui subclass: #MessageSetBrowser
       instanceVariableNames: 'collection methodSourceView'
       classVariableNames: ''
       poolDictionaries: ''
       category: nil
! 

!MessageSetBrowser class methodsFor: 'instance creation'!

on: aCollection label: aString
    ^super new init: aCollection label: aString
! !

!MessageSetBrowser methodsFor: 'callbacks'!

methodSelectedFrom: receiver at: anIndex
    | assoc |
    assoc _ receiver at: anIndex.
    methodSourceView contents: (assoc key "class" 
				    sourceCodeAt: assoc value "selector" ).
! !


!MessageSetBrowser methodsFor: 'private'!

init: aCollection label: aString
    | toplevel container list text labels label |
    collection _ aCollection.
    
    toplevel _ BWindow new: aString.
    container _ BContainer new: toplevel.

    labels _ OrderedCollection new: collection size.
    collection do: 
	[ :assoc | label _ (self classNameString: assoc key) ,
		       ' ', assoc value.
		   labels add: label ].

    list _ BList new: container.
    list height: 100; width: self browserWidth.
    list contents: labels elements: collection.
    list callback: self message: #methodSelectedFrom:at:.

    methodSourceView _ BText new: container.
    methodSourceView width: self browserWidth; height: 200.

    toplevel map.
!

browserWidth
    ^300
!

classNameString: anObject
    | name |
    name _ anObject name.
    name isNil
    	ifTrue: [ name _ anObject instanceClass name, ' class' ].
    ^name
! !



Gui subclass: #FirstBrowser
       instanceVariableNames: 'text methodList selectedClass showInherited 
			       container toplevel selectors sortedSelectors
			       showClassMethods browsedSelector'
       classVariableNames: 'ClassList'
       poolDictionaries: ''
       category: nil
! 


!FirstBrowser class methodsFor: 'instance creation'!

new
    ^super new init
! !


!FirstBrowser methodsFor: 'callbacks'!

classSelectedFrom: receiver at: anIndex
    | className index |
    className _ (receiver at: anIndex).
    (index _ className findFirst: [ :ch | ch ~~ Character space ]) > 0
	ifTrue: [ className _ className copyFrom: index to: className size ].

    selectedClass _ Smalltalk at: className asSymbol.

    self updateDisplay.
!

updateDisplay
    | class | 
    selectedClass notNil
	ifTrue: [ class _ selectedClass.
		  showClassMethods
		      ifTrue: [ class _ class class ].
		  showInherited
		      ifTrue: [ selectors _ class allSelectors ]
		      ifFalse: [ selectors _ class selectors ].
		  sortedSelectors 
		      ifTrue: [ selectors _ selectors asSortedCollection ].
		  methodList contents: selectors.
		  ].
!

updateDisplaySorted
    selectors notNil
	ifTrue: [ sortedSelectors 
		      ifTrue: [ methodList contents: selectors asSortedCollection ] ].
!


methodSelectedFrom: receiver at: anIndex
    | methodString class |

    browsedSelector _ receiver at: anIndex.
    showClassMethods
	ifTrue: [ class _ selectedClass class ]
	ifFalse: [ class _ selectedClass ].
    (class includesSelector: browsedSelector)
	ifTrue: [ methodString _ class sourceCodeAt: browsedSelector. ]
	ifFalse: [ methodString _ (class whichClassIncludesSelector: browsedSelector) 
		       sourceCodeAt: browsedSelector ].
    text contents: methodString.
!

showInherited: aBoolean
    showInherited _ aBoolean.
    self updateDisplay.
!

sortedSelectors: aBoolean
    sortedSelectors _ aBoolean.
    self updateDisplaySorted.
!

showInstanceMethods: aBoolean
    showClassMethods _ false.
    self updateDisplay.
!

showClassMethods: aBoolean
    showClassMethods _ true.
    self updateDisplay.
!

printSelection
   | selection | 
    "needs to compile a method in the context of the current class that
     does the evaluation, execute it, remove teh method, and print the result"
    text insertTextSelection: ' ', self doIt printString.
!

doIt
    | selection | 
    "needs to compile a method in the context of the current class that
     does the evaluation, execute it, remove teh method, and print the result"
    selection _ text getSelection.

    selection _ 'ObjectdoItMethod ^[ ', selection, ' ] value'.
    Object compileString: selection 
	   ifError: [ :file :line :error | stdout nextPutAll: 'error'; nl.
					     file printNl.
					      line printNl.
					      error  printNl.
					      ^nil ].
    ^nil ObjectdoItMethod.
!

compileIt
    | contents | 
    contents _ text contents.
    selectedClass notNil
	ifTrue: [ selectedClass compileString: contents ]
!

saveImage
    Smalltalk snapshot
!

inspectSelection
     self doIt vinspect.
   " ObjectInspector new:"
!

browseImplementors
    | implementors |
    browsedSelector isNil ifTrue: [ ^nil ].
    implementors _ OrderedCollection new.
    self allClassesDo: 
	[ :class | (class includesSelector:  browsedSelector)
		       ifTrue: [ implementors add: class -> browsedSelector].
		   ].
    MessageSetBrowser on: implementors label: browsedSelector, ' implementors'.
!

browseSenders
    | senders set |
    browsedSelector isNil ifTrue: [ ^nil ].
    senders _ OrderedCollection new.
    self allClassesDo: 
	[ :class | set _ class whichSelectorsReferTo: browsedSelector.
		   set do: 
		       [ :elem | senders add: class -> elem ].
	  ].
    MessageSetBrowser on: senders label: browsedSelector, ' senders'.
!

inspectClass
    | stream class name supername|
    
    selectedClass isNil ifTrue: [ ^nil ].


    class _ selectedClass.

    supername _ class superclass name.		"Handle the nil case"
    name _ class name.		"handle the nil case"

    stream _ WriteStream on: (String new: 50).
    stream nextPutAll: supername;
	space;
	nextPutAll: class kindOfSubclass;
	nextPut: $#;
	nextPutAll: name; nl.

    stream nextPutAll: '    instanceVariableNames: '''.
    class instVarNames do:
	[ :name | stream nextPutAll: name; space. ].
    stream nextPut: $'; nl.
    stream nextPutAll: '    classVariableNames: '''.
    class classVarNames do:
	[ :name | stream nextPutAll: name; space. ].
    stream nextPut: $'; nl.
    stream nextPutAll: '    poolDictionaries: '''.

    stream nextPut: $'; nl.
    stream nextPutAll: '    category: '''.
    stream print: class category; nextPut: $'; nl.

    text contents: stream contents.
!


createClass
    ^self notYetImplemented 
! !


!FirstBrowser methodsFor: 'private'!

init
    | color |
    showInherited _ false.
    sortedSelectors _ false.
    showClassMethods _ false.

    toplevel _ BWindow new: 'Class Browser'.
    container _ BContainer new: toplevel.
    color _ Blox createColor: 16rF000 green: 16rF000 blue: 16rFFFF.
    container backgroundColor: color.

    self createMenus.

    self createClassBrowser.

    self createSourceTextPane.
    self createInheritedToggle.
    self createSortedToggle.

    self createClassOrInstanceRadioGroup.

    self createTestMenuBar.

    HandleErrorsWithGui _ true.	"turn this on!"

    toplevel map.
!

createInheritedToggle
    | button |
    button _ BToggle new: container label: 'Show Inherited'.
    button callback: self message: 'showInherited:'.
!

createSortedToggle
    | button |
    button _ BToggle new: container label: 'Sort selectors'.
    button callback: self message: #sortedSelectors:.
!

createClassBrowser
    | subcontainer list collection |
    subcontainer _ BContainer new: container.
    subcontainer setVerticalLayout: false.
    list _ BList new: subcontainer.

    list numberOfColumns: 1; height: 100; width: 150.

    "list useVerticalLayout: true."

    "list contents: #(foo bar baz)."

    ClassList isNil
	ifTrue: [ ClassList  _ self hierarchyNames: Object ] .

    list contents: ClassList.

    list callback: self message: #classSelectedFrom:at:.
    (BPopupMenu new: list label: 'classPopup')
	callback: self using:
	    #((Inspect inspectClass)
		  ('Create' createClass)).

    methodList _ BList new: subcontainer.
    methodList numberOfColumns: 1; width: 200; height: 100.
    methodList callback: self message: #methodSelectedFrom:at:.

    self createMethodListMenu.
!


createMethodListMenu
    | menu |
    menu _ BPopupMenu new: methodList label: 'methodListPopup'.
    menu callback: self using:
	#(
	  ('Senders' browseSenders)
	      ('Implementors' browseImplementors)
	      ()
	      (Test2)
	      ).
!


createSourceTextPane
    (text _ BText new: container) height: 200; width: 400.
    text contents: '"Your method goes here"'. 

    self createSourceTextMenu.
!
    
createMenus
    | button color |
    BLabel new: container label: 'Menu:'.

    button _ BButton new: container label: 'quit'.
    button callback: button message: #quitPressed.
    color _ Blox createColor: 16rFFFF green: 16rD000 blue: 16rFFFF.
    button backgroundColor: color.

    button _ BButton new: container label: 'restart'.
    button callback: button message: 'restartPressed'.
    color _ Blox createColor: 16rFFFF green: 0 blue: 0.
    button backgroundColor: color.

    button _ BButton new: container label: 'Test error'.
    button callback: button message: #testError.
    color _ Blox createColor: 0 green: 16rFFFF blue: 0.
    button backgroundColor: color.

    "
    button _ BButton new: container label: 'help'.
    button callback: button message: 'helpPressed'.
    color _ Blox createColor: 16rFFFF green: 16r8000 blue: 16rFFFF.
    button backgroundColor: color.
    button enabled: false.

    button _ BButton new: container label: 'die'.
    button callback: button message: 'buttonPressed'.
    color _ Blox createColor: 16r7000 green: 16rF000 blue: 16rFFFF.
    button backgroundColor: color.
    button enabled: false.

    (BButton new: container  label: 'BURP!!!') enabled: false.
    "
!

createClassOrInstanceRadioGroup
    | subcontainer group ibutton cbutton |
    subcontainer _ BContainer new: container.
    subcontainer setVerticalLayout: false.

    BLabel new: subcontainer label: 'Show:'.
    group _ BRadioGroup new: subcontainer.
    ibutton _ BRadioButton new: group label: 'Instance'.
    ibutton callback: self message: #showInstanceMethods:.
    cbutton _ BRadioButton new: group label: 'Class'.
		cbutton posHoriz: ibutton. 
    cbutton callback: self message: #showClassMethods:.
!

createTestMenuBar
    | bar |
    bar _ BMenuBar new: container.

    self createFirstMenu: bar.
    self createSecondMenu: bar.
!

createSourceTextMenu
    | menu menuItem |
    menu _ BPopupMenu new: text label: 'sourceTextPopup'.
    menuItem _ BMenuItem new: menu label: 'DoIt'.
    menuItem callback: self message: #doIt.

    menuItem _ BMenuItem new: menu label: 'Compile it'.
    menuItem callback: self message: #compileIt.

    menuItem _ BMenuItem new: menu label: 'Inspect'.
    BMenuItem new: menu.
    menuItem callback: self message: #inspectSelection.

    menuItem _ BMenuItem new: menu label: 'Print'.
    menuItem callback: self message: 'printSelection'.
    
    BMenuItem new: menu.
    menuItem _ BMenuItem new: menu label: 'Debug'.

    BMenuItem new: menu.
    menuItem _ BMenuItem new: menu label: 'Save image'.
    menuItem callback: self message: #saveImage.
!

createFirstMenu: bar
    | menu menuItem |
    menu _ BMenu new: bar label: 'testIt'.
    menuItem _ BMenuItem new: menu label: 'a one'.

    menuItem callback: menuItem message: 'itemChosen'.

    menuItem _ BMenuItem new: menu label: 'and a two'.
    menuItem _ BMenuItem new: menu label: 'and away'.
    menuItem _ BMenuItem new: menu label: 'we go'.

    bar add: menu.
!

createSecondMenu: bar
    | menu menuItem |
    menu _ BMenu new: bar label: 'quem?'.

    menuItem _ BMenuItem new: menu label: 'testme '.
    menuItem callback: menuItem message: 'pickMeHarder'.

    menuItem _ BMenuItem new: menu label: 'and a two'.
    menuItem _ BMenuItem new: menu label: 'and away'.
    
    bar add: menu.
!

hierarchyNames: startingClass
    | collection |
    
    collection _ OrderedCollection new.
    self computeSubhierarhcyFrom: startingClass level: 0 into: collection.
    ^collection
!

computeSubhierarhcyFrom: startingClass level: level into: collection
    | name prefix |
    name _ startingClass name.
    name notNil ifTrue: 
	[ prefix _ String new: level.
	  1 to: level do: [ :i | prefix at: i put: Character space ].
	  collection add: prefix, name.
	  ].
	
    startingClass subclasses do:
	[ :subclass | self computeSubhierarhcyFrom: subclass 
			   level: level + 1
			   into: collection ].

!

allClassesDo: aBlock
    aBlock value: Object.
    Object allSubclassesDo: [ :class | aBlock value: class ].
! !


Gui subclass: #ObjectInspector
       instanceVariableNames: 'inspectedObject instVarList inspectionPane
			       instVarSelection'
       classVariableNames: ''
       poolDictionaries: ''
       category: nil
! 

!ObjectInspector class methodsFor: 'instance creation'!

new: anObject
    ^self new init: anObject
!

new: anObject in: aContainer
    ^self new init: anObject in: aContainer
!

new: container names: nameList elements: elements
    ^self new init: container names: nameList elements: elements
! !

!ObjectInspector methodsFor: 'callbacks'!

showInstVarFrom: receiver at: anIndex
    instVarSelection _ anIndex.
    inspectionPane contents: (instVarList at: anIndex) printString.
!

inspectSelection
    instVarSelection notNil
	ifTrue: [ ObjectInspector new: (instVarList at: instVarSelection) ].
! !


!ObjectInspector methodsFor: 'private'!

init: container names: nameList elements: elements
    | menu menuItem |

    instVarList _ BList new: container.
    instVarList width: 100; height: 100.

    instVarList contents: nameList elements: elements.
    instVarList callback: self message: #showInstVarFrom:at: .

    
    menu _ BPopupMenu new: instVarList label: 'ObjectInspectorPopup'.
    menuItem _ BMenuItem new: menu label: 'Inspect'.
    menuItem callback: self message: #inspectSelection.

    inspectionPane _ BText new: container.
    inspectionPane width: 200; height: 100.
    (BPopupMenu new: inspectionPane label: 'ObjectInspector')
	callback: self using:
	    #(('Do it' doIt)
		  ('Print it' printIt)
		  (Accept acceptIt)).

! 

init: anObject in: aContainer
    | class instanceValues instVarNames numInstVars numIndexVars totalVars |
    inspectedObject _ anObject.

    class _ anObject class.
    
    numIndexVars _ anObject basicSize. "scale this "
    numInstVars _ class instSize.

    totalVars _ numIndexVars + numInstVars.

    instVarNames _ OrderedCollection new: totalVars + 1.
    instVarNames add: #self.
    numInstVars > 0
	ifTrue: [ instVarNames addAllLast: class allInstVarNames. ].
    
    1 to: numIndexVars do:
	[ :i | instVarNames add: i printString ].

    instanceValues _ OrderedCollection new: totalVars + 1.
    instanceValues add: anObject.
    2 to: totalVars + 1 do:
	[ :i | instanceValues add: (anObject instVarAt: i - 1) ].

    self init: aContainer names: instVarNames elements: instanceValues.
!

init: anObject
    | topLevel container name |
    
    name _ anObject class name.
    name isNil ifTrue: [ name _ anObject name, ' class' ].

    topLevel _ BWindow new: name.
    container _ BContainer new: topLevel.
    container setVerticalLayout: false.

    self init: anObject in: container.

    topLevel map.
! 

originalinit: anObject
    | topLevel container class menu menuItem instanceValues instVarNames name 
      numInstVars numIndexVars totalVars |
    
    name _ anObject class name.
    name isNil ifTrue: [ name _ anObject name, ' class' ].

    inspectedObject _ anObject.

    topLevel _ BWindow new: name.
    container _ BContainer new: topLevel.
    container setVerticalLayout: false.

    instVarList _ BList new: container.
    instVarList width: 100; height: 100.

    class _ anObject class.
    
    numIndexVars _ anObject basicSize. "scale this "
    numInstVars _ class instSize.

    totalVars _ numIndexVars + numInstVars.

    instVarNames _ OrderedCollection new: totalVars + 1.

    instVarNames add: #self.
    numInstVars > 0
	ifTrue: [ instVarNames addAllLast: class allInstVarNames. ].
    
    1 to: numIndexVars do:
	[ :i | instVarNames add: i printString ].

    instanceValues _ OrderedCollection new: totalVars + 1.
    instanceValues add: anObject.
    2 to: totalVars + 1 do:
	[ :i | instanceValues add: (anObject instVarAt: i - 1) ].

    instVarList contents: instVarNames elements: instanceValues.
    instVarList callback: self message: #showInstVarFrom:at: .
    
    menu _ BPopupMenu new: instVarList label: 'ObjectInspectorPopup'.
    menuItem _ BMenuItem new: menu label: 'Inspect'.
    menuItem callback: self message: #inspectSelection.

    inspectionPane _ BText new: container.
    inspectionPane width: 200; height: 100.
    (BPopupMenu new: inspectionPane label: 'ObjectInspector')
	callback: self using:
	    #(('Do it' doIt)
		  ('Print it' printIt)
		  (Accept acceptIt)).

    topLevel map.
! 


doIt
    | selection | 
    "needs to compile a method in the context of the current class that
     does the evaluation, execute it, remove the method, and print the result"
    selection _ inspectionPane getSelection.

    selection _ 'ObjectdoItMethod ^[ ', selection, ' ] value'.
    Object compileString: selection 
	   ifError: [ :file :line :error | stdout nextPutAll: 'error'; nl.
					     file printNl.
					      line printNl.
					      error  printNl.
					      ^nil ].
    ^nil ObjectdoItMethod.
!

printIt
   | selection | 
    "needs to compile a method in the context of the current class that
     does the evaluation, execute it, remove teh method, and print the result"
    inspectionPane insertTextSelection: ' ', self doIt printString.
!

acceptIt
    | value |
    instVarSelection notNil ifTrue: 
	[ value _ self doIt.
	  inspectedObject instVarAt: instVarSelection put: value ].
! !



Gui subclass: #ContextInspector
       instanceVariableNames: 'contextHolder methodSource receiverVars'
       classVariableNames: ''
       poolDictionaries: ''
       category: nil
! 

!ContextInspector class methodsFor: 'instance creation'!

new: contextBrowser
    ^self new init: contextBrowser
! !

!ContextInspector methodsFor: 'private'!

init: contextBrowser
    | toplevel container text subcontainer list class selector context |
    contextHolder _ contextBrowser.

    context _ contextHolder currentContext.

    selector _ context selector.
"brd 7/16 - exit if selector is nil"
    selector isNil ifTrue: ['Unable to inspect method context' printNl. ^self].

    toplevel _ BWindow new: 'Context Inspector'.
    container _ BContainer new: toplevel.
    
    text _ BText new: container.
    text width: 450; height: 200.
    
    context _ contextHolder currentContext.

    selector _ context selector.
    class _ (context receiver class includesSelector: selector)
	ifTrue: [ context receiver class]
	ifFalse: [  context receiver class 
			whichClassIncludesSelector: selector ].
			
    text contents: (class sourceCodeAt: selector).

    subcontainer _ BContainer new: container.
    subcontainer setVerticalLayout: false.
    
    ObjectInspector new: contextHolder currentContext receiver 
		    in: subcontainer.

    self addStackInspector: subcontainer.


    toplevel map.
!



addStackInspector: aContainer
    | context stackVars stackVarValues method totalVars |
    
    context _ contextHolder currentContext.
    
    method _ self getMethodFromContext: context.

    totalVars _ method numArgs + method numTemps.

    stackVars _ OrderedCollection new: totalVars + 1.
    stackVars add: #thisContext.

    1 to: totalVars do:
	[ :i | stackVars add: i printString ].
    
    stackVarValues _ OrderedCollection new: totalVars + 1.
    stackVarValues add: context.

    2 to: totalVars + 1 do:
	[ :i | stackVarValues add: (context at: i - 1) ].

    ObjectInspector new: aContainer
		    names: stackVars elements: stackVarValues.
!

getMethodFromContext: context
    "limit for right now to just method contexts"
    ^context method.
! !



Gui subclass: #ErrorInspector
       instanceVariableNames: 'callstackList currentSelection numContexts'
       classVariableNames: ''
       poolDictionaries: ''
       category: nil
! 



!ErrorInspector class methodsFor: 'instance creation'!

new: message names: contextStrings contexts: contexts
    ^self new init: message names: contextStrings contexts: contexts
! !

!ErrorInspector methodsFor: 'accessing'!

currentContext
    currentSelection isNil ifTrue: [ ^nil ].
    ^callstackList at: currentSelection
!

next
    currentSelection isNil ifTrue: [ ^nil ].
    currentSelection _ currentSelection + 1 min: numContexts.
    "Should change the currently selected list item"
!

previous
    currentSelection isNil ifTrue: [ ^nil ].
    currentSelection _ currentSelection - 1 max: 1.
    "Should change the currently selected list item"
! !


!ErrorInspector methodsFor: 'callback'!

inspectContext
    ContextInspector new: self 
!

contextSelectedFrom: receiver at: anIndex
    currentSelection _ anIndex
! !


!ErrorInspector methodsFor: 'private'!

init: message names: nameList contexts: contexts
    | topLevel |
    numContexts _ contexts size.
    topLevel _ BWindow new: message.
    callstackList _ BList new: topLevel.
    callstackList width: 300; height: 100.
    
    callstackList contents: nameList elements: contexts.
    callstackList callback: self message: #contextSelectedFrom:at:.
    (BPopupMenu new: callstackList label: 'ErrorBacktrace')
	callback: self using:
	    #((Proceed)
		  (Exit)
		  ()
		  (Inspect inspectContext)).

    topLevel map.
! !


(Object includesSelector: #originalDoesNotUnderstand: )
    ifFalse: [ Object addSelector: #originalDoesNotUnderstand: 
		      withMethod: (Object compiledMethodAt: #doesNotUnderstand:) ]!

(Object includesSelector: #originalError: )
    ifFalse: [ Object addSelector: #originalError: 
		      withMethod: (Object compiledMethodAt: #error:) ]!

Smalltalk at: #BBrowserInitialized put: false!

!Object methodsFor: 'overriding'!

error: message
    | context names contexts name nameStr |
    "Check for blox init being called already!"
    HandleErrorsWithGui
	ifFalse: [ ^self originalError: message ].

    names _ OrderedCollection new.
    contexts _ OrderedCollection new.
    
    context _ thisContext.
	
    HandleErrorsWithGui _ false. "prevent infinite recursion"
    [ context notNil ] whileTrue: 
	[ nameStr _ WriteStream on: (String new: 5).
	  nameStr nextPutAll: context realReceiver;
	      nextPutAll: '>>#'; 
	      nextPutAll: context selector printString.
	  name _ nameStr contents.
	  names add: name.
	  contexts add: context.
	  context _ context parentContext.
	  ].
    
    ErrorInspector new: message names: names contexts: contexts.
    HandleErrorsWithGui _ true.
!

doesNotUnderstand: message
    (BBrowserInitialized & HandleErrorsWithGui )
	ifTrue: [ self error: 'Did not understand ', message selector. ]
	ifFalse: [ self originalDoesNotUnderstand: message ].
!

vinspect
    ObjectInspector new: self.
! !


!BlockContext methodsFor: 'debugging'!

callers
    self inspect.
    caller notNil
	ifTrue: [ caller callers ]
!

parentContext
    ^caller
!

selector
    ^selector
!

receiverName
    ^'[] in ', home class name
!

realReceiver
    "Temp hack"
    ^'[] in ', home class name

! !

!MethodContext methodsFor: 'debugging'!

callers
    self inspect.
    sender notNil 
	ifTrue: [ sender callers ]
!

parentContext
    ^sender
!

receiverName
    | name |
    name _ receiver class name.
    name isNil ifTrue: [ name _ receiver name ].
    ^name
!

realReceiver
    | name receiverClass methodClassName |
    
    receiverClass _ receiver class.
    receiverClass ~~ methodClass
	ifTrue: [ name _ self receiverName.
		  methodClassName _ methodClass name.
		  ^name , '(', methodClassName, ')'  ]
	ifFalse: [ ^self receiverName ].
! !

Smalltalk snapshot!

"

| x |
    KeepBrowserRunning _ true.
    BBrowserInitialized _ false.
    [ KeepBrowserRunning ] whileTrue: 
	[ 
	  Blox bloxInit.
	  FirstBrowser new.
          BBrowserInitialized _ true.
	  Blox dispatchEvents.
	  'dispatch events returning' printNl. ].
! 
"


