
STAppDelegate subclass: #AppDelegate
		 instanceVariableNames: 'mainMenu browserController app workspace console'
		 classVariableNames: ''
		 poolDictionaries: ''
		 category: nil
!

AppDelegate comment: 'Creates everything after the Application has been launched'!

!AppDelegate methodsFor: 'App Delegation'!

appDidInit: arg0
	app _ Application new.
		
	self initMainMenu.
	browserController _ ClassBrowserController alloc init.
	workspace _ WorkspaceController alloc init.
	console _ ConsoleController alloc init.
!

appWillTerminate: sender
	Smalltalk quitPrimitive.

!!

!AppDelegate methodsFor: 'Initialising the Interface'!

initMainMenu
	| menuItem services windows info system |
		
	mainMenu _ Menu alloc initTitle: 'Smalltalk'.
	
	menuItem _ mainMenu addItem: 'Info' action: nil keyEquivalent: nil.
	mainMenu setSubmenu: ((self initInfoMenu) id) forItem: (menuItem id).
	menuItem _ mainMenu addItem: 'System' action: nil keyEquivalent: nil.
	mainMenu setSubmenu: ((self initSystemMenu) id) forItem: (menuItem id).
	menuItem _ mainMenu addItem: 'Edit' action: nil keyEquivalent: nil.
	mainMenu setSubmenu: ((self initEditMenu) id) forItem: (menuItem id).
	menuItem _ mainMenu addItem: 'Tools' action: nil keyEquivalent: nil.
	mainMenu setSubmenu: ((self initToolMenu) id) forItem: (menuItem id).

	"
	menuItem _ mainMenu addItem: 'Windows' action: nil keyEquivalent: nil.
	windows _ Menu alloc initTitle: 'Windows'.
	windows addItem: 'Arrange in Front' action: (OCInterface getUid: 'arrangeInFront:') keyEquivalent: nil.
	windows addItem: 'Miniaturize Window' action: (OCInterface getUid: 'miniaturize:') keyEquivalent: ($m asciiValue).
	windows addItem: 'Close Window' action: (OCInterface getUid: 'performClose:') keyEquivalent: ($W asciiValue).
	mainMenu setSubmenu: (windows id) forItem: (menuItem id).
	app setWindowsMenu: windows id.
	"
	menuItem _ mainMenu addItem: 'Services' action: nil keyEquivalent: nil.
	services _ Menu alloc initTitle: 'Services'.
	mainMenu setSubmenu: (services id) forItem: (menuItem id).
	app setServicesMenu: (services id).

	menuItem _ mainMenu addItem: 'Print...' action: nil keyEquivalent: ($p asciiValue).
	menuItem _ mainMenu addItem: 'Hide' action: (OCInterface getUid: 'hide:') keyEquivalent: ($h asciiValue).
	menuItem _ mainMenu addItem: 'Quit' action: (OCInterface getUid: 'terminate:') keyEquivalent: ($q asciiValue).	
	
	app setMainMenu: mainMenu id.
!
			
initInfoMenu
	| infoMenu |
	infoMenu _ Menu alloc initTitle: 'Info'.
	infoMenu addItem: 'Info...' action: nil keyEquivalent: nil.
	infoMenu addItem: 'Preferences...' action: nil keyEquivalent: nil.
	infoMenu addItem: 'Help...' action: (OCInterface getUid: 'showHelpPanel:') keyEquivalent: ($? asciiValue).
	^infoMenu	
!
		
initSystemMenu
	| systemMenu menuItem |
	systemMenu _ Menu alloc initTitle: 'System'.
	systemMenu addItem: 'Snapshot...' action: (OCInterface getUid: 'action3:') keyEquivalent: ($s asciiValue).
	systemMenu addItem: 'File In...' action: (OCInterface getUid: 'action4:') keyEquivalent: ($i asciiValue).
	systemMenu addItem: 'File Out...' action: (OCInterface getUid: 'action5:') keyEquivalent: ($o asciiValue).
	^systemMenu	
!
		
initEditMenu
	| editMenu |
	editMenu _ Menu alloc initTitle: 'Edit'.
	" Don't give them menuitems a specific target that way they will
	  be forwarded to first responder "
	editMenu addItem: 'Cut' action: (OCInterface getUid: 'cut:') keyEquivalent: ($x asciiValue).
	editMenu addItem: 'Copy' action: (OCInterface getUid: 'copy:') keyEquivalent: ($c asciiValue).
	editMenu addItem: 'Paste' action: (OCInterface getUid: 'paste:') keyEquivalent: ($v asciiValue).
	^editMenu	
!
		
initToolMenu
	| toolMenu menuItem |
	toolMenu _ Menu alloc initTitle: 'Edit'.
	menuItem _ toolMenu addItem: 'Workspace...' action: (OCInterface getUid: 'action1:') keyEquivalent: ($W asciiValue).
	menuItem setTarget: self id.
	menuItem _ toolMenu addItem: 'Console...' action: (OCInterface getUid: 'action2:') keyEquivalent: ($C asciiValue).
	menuItem setTarget: self id.
	toolMenu addItem: 'File Browser...' action: nil keyEquivalent: ($f asciiValue).
	^toolMenu		
!!

!AppDelegate methodsFor: 'Action'!

action1: sender
	| panel |
	panel _ workspace panel.
	panel makeKeyAndOrderFront: nil.
	panel display.
!

action2: sender
	| panel |
	panel _ console panel.
	panel makeKeyAndOrderFront: (panel id).
	panel display.
!

action3: sender
	" Called by menu item to save the image "
	Smalltalk snapshot.
!

action4: sender
	" Action for filing in "
!

action5: sender
	" Action for filing out "
	| savePanel value file dir class rect |

	rect _ Rect alloc initFromX: 200.0 y: 200.0 width: 500.0 height: 400.0.

	" For some reason #new won't work "
	savePanel _ SavePanel newContent: (rect asCPointer)
							   style: 6
							 backing: 2
						  buttonMask: (4 bitOr: 1)
							   defer: YES.
							   
	savePanel setRequiredFileType: 'st'.
	savePanel setTitle: 'Filing Out'.
	value _ savePanel runModal. "ForDirectory: '/' file: 'AClass.st' ."

	value == 1
		ifTrue: [
			file _ savePanel filename. 
			dir _ savePanel directory.
			dir, '/', file printNl. ].

!!

