"======================================================================
|
|   X graphic context class
|
|   $Revision: 1.3 $
|   $Date: 1995/06/06 14:56:15 $
|   $Author: sbb $
|
 ======================================================================"

"======================================================================
|
| Copyright (C) 1990, 1991, 1992, 1994, 1995 Free Software Foundation, Inc.
| Written by Steve Byrne.
|
| This file is part of GNU Smalltalk.
|
| GNU Smalltalk is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 1, or (at your option) any later version.
| 
| GNU Smalltalk is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| GNU Smalltalk; see the file COPYING.  If not, write to the Free Software
| Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
|
 ======================================================================"


"
|     Change Log
| ============================================================================
| Author       Date       Change 
| sbb	     26 Oct 90	  Began rehacking
|
| sbyrne     24 May 90	  created.
|
"

Fontable subclass: #GC
	 instanceVariableNames: ''
	 classVariableNames: ''
	 poolDictionaries: ''
	 category: 'X Objects'
!

Smalltalk at: #GCMasks put: Dictionary new!

| mask |
    "Set up the bit masks into a dictionary for use by the copyGC: operation"
    mask _ 1.
    #(Function
	  PlaneMask
	  Foreground
	  Background
	  LineWidth
	  LineStyle
	  CapStyle
	  JoinStyle
	  FillStyle
	  FillRule
	  Tile
	  Stipple
	  TileStippleXOrigin
	  TileStippleYOrigin
	  Font
	  SubwindowMode
	  GraphicsExposures
	  ClipXOrigin
	  ClipYOrigin
	  DashOffset
	  Dashes
	  ArcMode) do: 
	[ :name | GCMasks at: name
			  put: mask.
		  mask _ mask bitShift: 1 ].
    "Mask is larger than the largest bit.  Subtracting 1 turns all bits 
     that correspond to existing masks on."
    GCMasks at: #All put: mask - 1.
!


!GC class methodsFor: 'X object instance creation'!

createGC: drawable onServer: server args: aBlock
    | id req |
    id _ server nextId.
    req _ XGCRequest command: 55.
    req ulong: id; ulong: drawable id.
    aBlock notNil
	ifTrue: [ aBlock value: req ]
	ifFalse: [ req noBits ].
    req queueOn: server.
    ^self new init: id onServer: server
! !


!GC class methodsFor: 'useful functions'!

withGC: drawable args: aBlock do: bodyBlock
    | gc |
    gc _ self createGC: drawable onServer: drawable server args: aBlock.
    bodyBlock value: gc.
    gc freeGC.
! !


!GC methodsFor: 'X protocol request'!

changeGC: aBlock
    | req |
    req _ XGCRequest command: 56.
    req ulong: id.
    aBlock notNil
	ifTrue: [ aBlock value: req ]
	ifFalse: [ req noBits ].
    req queueOn: server
!

copyGC: srcGC valueMasks: anArray
    | req mask |
    req _ XRequest fixedCommand: 57.
    req ulong: srcGC id; ulong: id.
    mask _ 0.
    anArray notNil ifTrue:
	[ anArray do: [ :sym | mask _ mask bitOr: (GCMasks at: sym) ] ].
    req ulong: mask.
    req queueOn: server
!

freeGC
    | req |
    req _ XRequest fixedCommand: 60.
    req ulong: id.
    req queueOn: server
!

setClipRectangles: clipXOrigin clipYOrigin: clipYOrigin rectangles: rectangles
    ordering: ordering
    | req |
    req _ XRequest command: 59
		   aux: (XProtocol map: ordering 
				   into: #(UnSorted YSorted 
					       YXSorted YXBanded)).
    req ulong: id; word: clipXOrigin; word: clipYOrigin.
    rectangles do: [ :rect | req rectangle: rect ].
    req queueOn: server
!

setDashes: dashOffset dashes: dashes
    | req |
    req _ XRequest command: 58.
    req ulong: id; uword: dashOffset; uword: dashes size; bytes: dashes.
    req queueOn: server
! !

LoadStatus _ 'GC'!






