"======================================================================
|
|   X server class.  
|   The primary connection to the X server is mediated through this class.
|
|   $Revision: 1.4 $
|   $Date: 1995/06/06 14:56:15 $
|   $Author: sbb $
|
 ======================================================================"

"======================================================================
|
| Copyright (C) 1990, 1991, 1992, 1994, 1995 Free Software Foundation, Inc.
| Written by Steve Byrne.
|
| This file is part of GNU Smalltalk.
|
| GNU Smalltalk is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 1, or (at your option) any later version.
| 
| GNU Smalltalk is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| GNU Smalltalk; see the file COPYING.  If not, write to the Free Software
| Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
|
 ======================================================================"

"
|     Change Log
| ============================================================================
| Author       Date       Change 
| sbb	     20 Sep 94	  Incorporated Joerg Rade's changes. (move newServer
|			  into the Server class, and move ServerConn to be a
|			  class variable.
|
| sbb	     23 Feb 92	  Added hooks for X authority.
|			  Added connectToDisplay for parameterless
|			  operation.
|
| sbb	     16 Feb 92	  created a while ago.
|
"

"??? should we reference server? or self?"

XServerObject subclass: #Server
	  instanceVariableNames: 'nextId root socket objects queue pos
				  sendQueue doTiming' 
				"Pos is a mod 4 counter...can go higher, but
				 only used for its value mod 4."
	  classVariableNames: 'ByteBuffer ServerConn'
	  poolDictionaries: ''
	  category: 'X Objects'
!

Behavior defineCFunc: 'connectToServer'
	 withSelectorArgs: 'connectToServer: hostName display: displayNum'
	 forClass: Server
	 returning: #int
	 args: #(string int)
!

Behavior defineCFunc: 'waitForSocket'
	 withSelectorArgs: 'waitForSocket: socket timeOut: anInteger'
	 forClass: Server
	 returning: #int	"need a boolean type"
	 args: #(int int)
!


Smalltalk at: #ServerDebugging put: false!



(Smalltalk includesKey: #Window)
    ifFalse: [ Smalltalk at: #Window put: nil ]
!

(Smalltalk includesKey: #Visual)
    ifFalse: [ Smalltalk at: #Visual put: nil ]
!

(Smalltalk includesKey: #Colormap)
    ifFalse: [ Smalltalk at: #Colormap put: nil ]
!

(Smalltalk includesKey: #Atom)
    ifFalse: [ Smalltalk at: #Atom put: nil ]
!


!Server class methodsFor: 'instance creation'!

connectTo: hostName display: anInteger
    ^self new init: hostName display: anInteger
!

connectToDisplay
    | disp index host displayNumber |
    disp _ Smalltalk getenv: 'DISPLAY'.
    disp isNil 
	ifTrue: [ ^self error: 'environment variable DISPLAY is not set' ].
    index _ disp indexOf: $:
		 ifAbsent: [ ^self error: 'malformed DISPLAY variable' ].
    host _ disp copyFrom: 1 to: index - 1.
    displayNumber _ (disp copyFrom: index + 1 to: disp size) asInteger.
    ^self connectTo: host display: displayNumber
!

newServer
    | s disp index host displayNumber d | 
    ServerConn notNil ifTrue: [ServerConn close].
    disp _ Smalltalk getenv: 'DISPLAY'.
    disp isNil 
	ifTrue: [ ^self error: 'environment variable DISPLAY is not set' ].
    index _ disp indexOf: $:
		 ifAbsent: [ ^self error: 'malformed DISPLAY variable' ].
    host _ disp copyFrom: 1 to: index - 1.
    displayNumber _ (disp copyFrom: index + 1 to: disp size) asInteger.
    ServerConn _ s _ Server connectTo: host display: displayNumber.
    s isNil ifTrue: [ ^self error: 'Failed to create display' ].

    ^s
! !



!Server class methodsFor: 'initialization'!

initialize
    ByteBuffer _ ByteArray new: 4 "large enough to hold longs".
    ServerConn _ nil.		"this may not be exactly the right thing"
!!



!Server methodsFor: 'X request primitive access'!

bool
    ^self ubyte			" ### more complex later"
!

byte
    | val |
    socket read: ByteBuffer numBytes: 1.
    pos _ pos + 1.
    val _ ByteBuffer at: 1.
    val > 128 
	ifTrue: [ ^val negated ].
    ^val
!

ubyte
    socket read: ByteBuffer numBytes: 1.
    pos _ pos + 1.
    ^ByteBuffer at: 1
!

word
    | val |
    socket read: ByteBuffer numBytes: 2.
    pos _ pos + 2.
    Bigendian
	ifTrue: [ val _ (ByteBuffer at: 1) * 256 + (ByteBuffer at: 2) ]
	ifFalse: [ val _ (ByteBuffer at: 2) * 256 + (ByteBuffer at: 1) ].
    val > 16r7FFF ifTrue: [ ^val - 16r10000].
    ^val
!

uword
    socket read: ByteBuffer numBytes: 2.
    pos _ pos + 2.
    Bigendian
	ifTrue: [ ^(ByteBuffer at: 1) * 256 + (ByteBuffer at: 2) ]
	ifFalse: [ ^(ByteBuffer at: 2) * 256 + (ByteBuffer at: 1) ]
!

long
    | val hiByte |
    socket read: ByteBuffer numBytes: 4.
    Bigendian
	ifTrue: [ hiByte _ ByteBuffer at: 1.
		  val _ (hiByte bitAnd: 16r3F) * 16r1000000 +
		      ((ByteBuffer at: 2) * 16r10000) +
		      ((ByteBuffer at: 3) * 16r100) +
		      (ByteBuffer at: 4) ]
	ifFalse: [ hiByte _ ByteBuffer at: 4.
		  val _ (hiByte bitAnd: 16r3F) * 16r1000000 +
		      ((ByteBuffer at: 3) * 16r10000) +
		      ((ByteBuffer at: 2) * 16r100) +
		      (ByteBuffer at: 1) ].
    hiByte > 128
	ifTrue: [ ^val negated ].
    ^val
!

ulong
    | hiByte |
    socket read: ByteBuffer numBytes: 4.
    Bigendian
	ifTrue: [ hiByte _ (ByteBuffer at: 1) bitAnd: 16r3F.
		  ^hiByte * 16r1000000 +
		      ((ByteBuffer at: 2) * 16r10000) +
		      ((ByteBuffer at: 3) * 16r100) +
		      (ByteBuffer at: 4) ]
	ifFalse: [ hiByte _ (ByteBuffer at: 4) bitAnd: 16r3F.
		  ^hiByte * 16r1000000 +
		      ((ByteBuffer at: 3) * 16r10000) +
		      ((ByteBuffer at: 2) * 16r100) +
		      (ByteBuffer at: 1) ]
!

bytes: byteArray
    (socket write: byteArray) < 0
	ifTrue: [ ^self error: 'failed to write ', 
		      (byteArray size printString), ' bytes' ]
!	      

getBytes: nBytes
    | bytes |
    bytes _ ByteArray new: nBytes.
    (socket read: bytes) < 0
	ifTrue: [ ^self error: 'failed to get ', (nBytes printString), 
		      ' bytes'].
    pos _ pos + nBytes.
    ^bytes
!

getString: nChars
    | str | 
    str _ String new: nChars.
    (socket read: str numChars: nChars) < 0
	ifTrue: [ ^self error: 'failed to get string ', (nChars printString), 
		      ' chars long' ].
    pos _ pos + nChars.
    ^str
!

skip: nBytes
    self getBytes: nBytes
!

str 
    | len |
    "Gets a counted string from the stream...apparently no padding"
    len _ self ubyte.
    pos _ len + 1.		"account for the len byte"
    ^socket readString: len
!

skipPad
    | bytes |
    "skips to a 4 byte boundary"
    
    bytes _ 4 - (pos bitAnd: 3).
    (bytes < 4) & (bytes > 0)
	ifTrue: [ self getBytes: bytes ]. "this could be more efficient"
    pos _ 0
!

getUnpaddedString: nChars
    ^socket readString: nChars
!

keymap
    ^self getUnpaddedString: 31
!

keymap: aByteArray
    ^self bytes: aByteArray
!

close
    socket close
!

nextEvent
    | firstByte error event |
    queue isEmpty
	ifFalse: [ ^queue removeFirst ].
    [ (firstByte _ self ubyte) == 0 ]
	whileTrue: [ error _ XError newFrom: self.
		     error inspect ].
    event _ XEvent newFrom: self type: firstByte.
    ^event
!
    
queueEvent: event
    queue addLast: event
!


getReply
    | "love at" firstByte error event auxByte |
    [ (firstByte _ self ubyte) ~= 1 ] "suck up events and errors"
	whileTrue: [ firstByte == 0
			 ifTrue: [ error _ XError newFrom: self.
				   error  inspect ]
			 ifFalse: [ event _ XEvent newFrom: self
						   type: firstByte.
				    self queueEvent: event ]
			 ].
    auxByte _ self ubyte.
    self word.			"sequence number"
    self long.			"skip the length"
    pos _ 0.
    ^auxByte
!

getMappedReply: symbolArray
    | firstByte |
    firstByte _ self getReply.
    firstByte < symbolArray size
	ifTrue: [ ^symbolArray at: firstByte + 1 ]
	ifFalse: [ self error: 'Reply type out of bounds: ', 
		       (firstByte printString) ]
!    

getReplyAndLength: aBlock
    | firstByte auxByte len event error |
    [ (firstByte _ self ubyte) ~= 1 ] "suck up events and errors"
	whileTrue: [ firstByte == 0
			 ifTrue: [ error _ XError newFrom: self.
				   error  inspect ]
			 ifFalse: [ event _ XEvent newFrom: self
						   type: firstByte.
				    self queueEvent: event ]
			 ].
    auxByte _ self ubyte.
    self word.			"sequence number"
    len _ self long.			"skip the length"
    pos _ 0.
    aBlock value: len value: auxByte
!

"
""maybeGetId: symbolArray 
""    | id |
""    id _ self ulong.
""    id < symbolArray size
""	ifTrue: [ ^symbolArray at: id + 1 ]
""	ifFalse: [ ^objects at: id ]
""!
""
""getId
""    ^objects at: self ulong
""!
"

host
    | family addr addrLen |
    family _ XProtocol unmap: (self ubyte) into: #(Internet DECnet Chaos 
						       Dragnet "chuckle").
    self ubyte.			"skip"
    addrLen _ self uword.
    addr _ self getBytes: addrLen.
    self skipPad.
    ^Host family: family address: addr
!

rootInit
    | majorVersion minorVersion releaseNum resourceIdBase resourceIdMask
      motionBufferSize maxRequestLen imageByteOrder bitmapBitOrder 
      bitmapFormatScanUnit bitmapFormatScanPad minKeycode maxKeycode vendor
      formats screens vendorLen numFormats numScreens | 

    majorVersion _ self uword.
    minorVersion _ self uword.
    self uword.		"skip length"

    releaseNum _ self ulong.
    resourceIdBase _ self ulong.
    resourceIdMask _ self ulong.

    nextId _ resourceIdBase.

    motionBufferSize _ self ulong.
    vendorLen _ self uword.
    maxRequestLen _ self uword.
    numScreens _ self ubyte.
    numFormats _ self ubyte.

    imageByteOrder _ self ubyte. "decode in LSBFirst or MSBFirst"
    bitmapBitOrder _ self ubyte. "decode into LeastSignificant or Most..."
    bitmapFormatScanUnit _ self ubyte.
    bitmapFormatScanPad _ self ubyte.
    minKeycode _ self ubyte.
    maxKeycode _ self ubyte.
    self long.			"ignored "
    
    vendor _ self getString: vendorLen.    
    self skipPad.

    formats _ Array new: numFormats.
    screens _ Array new: numScreens.

    1 to: numFormats do: 
	[ :i | formats at: i put: (self format) ].
    1 to: numScreens do: 
	[ :i | screens at: i put: (self screen) ].

    

    ^InitializeReply majorVersion: majorVersion minorVersion: minorVersion
		     releaseNum: releaseNum resourceIdBase: resourceIdBase 
		     resourceIdMask: resourceIdMask
		     motionBufferSize: motionBufferSize
		     maxRequestLen: maxRequestLen 
		     imageByteOrder: imageByteOrder 
		     bitmapBitOrder: bitmapBitOrder 
		     bitmapFormatScanUnit: bitmapFormatScanUnit 
		     bitmapFormatScanPad: bitmapFormatScanPad 
		     minKeycode: minKeycode maxKeycode: maxKeycode
		     vendor: vendor formats: formats screens: screens
!

    


format
    | depth bitsPerPixel scanlinePad |
    depth _ self ubyte.
    bitsPerPixel _ self ubyte.
    scanlinePad _ self ubyte.
    self skip: 5.
    ^Format depth: depth bitsPerPixel: bitsPerPixel scanlinePad: scanlinePad
!

screen
    | window defaultColormap whitePixel blackPixel currentInputMasks
      widthInPixels heightInPixels widthInMM heightInMM minInstalledMaps
      maxInstalledMaps rootVisual backingStores saveUnders rootDepth
      allowedDepths numDepths |
    window _ Window idFrom: self.
    defaultColormap _ Colormap idFrom: self.
    whitePixel _ self ulong.
    blackPixel _ self ulong.
    currentInputMasks _ self ulong.
    widthInPixels _ self uword.
    heightInPixels _ self uword.
    widthInMM _ self uword.
    heightInMM _ self uword.
    minInstalledMaps _ self uword.
    maxInstalledMaps _ self uword.
    rootVisual _ Visual idFrom: self.
    backingStores _ XProtocol unmap: (self ubyte) 
			      into: #(Never WhenMapped Always).
    saveUnders _ self ubyte"bool".
    rootDepth _ self ubyte.
    numDepths _ self ubyte.
    allowedDepths _ Array new: numDepths.
    1 to: numDepths do:
	[ :i | allowedDepths at: i put: self depth ].
    ^Screen window: window defaultColormap: defaultColormap 
	    whitePixel: whitePixel blackPixel: blackPixel
	    currentInputMasks: currentInputMasks widthInPixels: widthInPixels
	    heightInPixels: heightInPixels widthInMM: widthInMM
	    heightInMM: heightInMM minInstalledMaps: minInstalledMaps
	    maxInstalledMaps: maxInstalledMaps rootVisual: rootVisual
	    backingStores: backingStores saveUnders: saveUnders
	    rootDepth: rootDepth allowedDepths: allowedDepths
!

depth
    | depth visuals numVisuals |
    depth _ self ubyte.
    self ubyte.
    numVisuals _ self uword.
    self ulong.			"ignored"
    visuals _ Array new: numVisuals.
    1 to: numVisuals do:
	[ :i | visuals at: i put: server visualType ].
    ^Depth depth: depth visuals: visuals
!

visualType
    | visualId class bitsPerRGB colormapEntries redMask greenMask blueMask |
    visualId _ Visual idFrom: self.
    class _ XProtocol unmap: self ubyte 
		      into: #(StaticGray GrayScale StaticColor PseudoColor
				  TrueColor DirectColor).
    bitsPerRGB _ self ubyte.
    colormapEntries _ self uword.
    redMask _ self ulong.
    greenMask _ self ulong.
    blueMask _ self ulong.
    self ulong.			"ignored"

    ^VisualType visualId: visualId class: class bitsPerRGB: bitsPerRGB
		colormapEntries: colormapEntries redMask: redMask
		greenMask: greenMask blueMask: blueMask
!

charInfo
    | leftSideBearing rightSideBearing characterWidth ascent descent
      attributes |
    leftSideBearing _ self word.
    rightSideBearing _ self word.
    characterWidth _ self word.
    ascent _ self word.
    descent _ self word.
    attributes _ self uword.
    ^CharInfo leftSideBearing: leftSideBearing 
	      rightSideBearing: rightSideBearing
	      characterWidth: characterWidth ascent: ascent descent: descent
	      attributes: attributes
!


fontProp
    | name value |
    name _ Atom idFrom: self.
    value _ self ulong.
    ^FontProp name: name value: value
!

rgb
    | red green blue |
    red _ self uword.
    green _ self uword.
    blue _ self uword.
    self uword.			"unused"
    ^RGB red: red green: green blue: blue
!

timeCoord
    | time x y |
    time _ self ulong.
    x _ self uword.
    y _ self uword.
    ^TimeCoord time: time x: x y: y
! !



!Server methodsFor: 'X Request queue management'!

queueRequest: bytes
    "This method decides how requests are handled.  It can either queue them
    or write them immediately to the socket."
    ServerDebugging ifTrue: [ bytes printNl ].
    doTiming isNil
	ifTrue: [ socket write: bytes ]
	ifFalse: [ "1 to: 100 do: [ :i | socket write: bytes]" ].
!

doTiming: aValue
    doTiming _ aValue
!

flushQueue
    socket write: sendQueue
!!



!Server methodsFor: 'X object support'!

registerId: id as: xObject
    objects at: id put: xObject 
!

nextId
    ^nextId _ nextId + 1
!

root
    ^root

!!




!Server methodsFor: 'X protocol requests'!

allowEvents: mode time: time
    | req |
    req _ XRequest fixedCommand: 35 
		   aux: (XProtocol map: mode 
				   into: #(AsyncPointer SyncPointer 
					       ReplayPointer AsyncKeyboard
					       SyncKeyboard ReplayKeyboard
					       AsyncBoth SyncBoth)).
    req ulong: (XProtocol maybeMap: time into: #(CurrentTime)).
    req queueOn: server
!

bell: percent
    | req |
    req _ XRequest fixedCommand: 104 
		   aux: percent.
    req queueOn: server.
!

changeActivePointerGrab: eventMask cursor: cursorVar time: time
    | req |
    req _ XRequest fixedCommand: 30.
    req ulong: (XProtocol maybeMapId: cursorVar into: #(None)).
    req ulong: (XProtocol maybeMap: time into: #(CurrentTime)).
    req uword: (XProtocol mask: eventMask using: PointerEventNames).
    req queueOn: server
!

changeHosts: mode host: hostVar
    | req |
    req _ XRequest command: 109
		   aux: (XProtocol map: mode into: #(Insert Delete)).
    "Internet only for right now"
    req byte: 0.		"Internet"
    req byte: 0; uword: 4.	"internet addresses are always 4 bytes"
    hostVar do:
	[ :byte | req byte: byte ].
    req queueOn: server
!

changeKeyboardControl: aBlock
    | req |
    req _ XKeyboardRequest command: 102.
    aBlock notNil
	ifTrue: [ aBlock value: req ]
	ifFalse: [ req noBits ].
    req queueOn: server
!

changeKeyboardMapping: firstKeycode keysymsPerKeycode: perKeycode 
    keysyms: keysyms
    | req |
    req _ XRequest command: 100
		   aux: (keysyms size // perKeycode).
    req ubyte: firstKeycode; byte: perKeycode; word: 0.	"unused"
    keysyms do:
	[ :keysym | req keysym: keysym ].
    req queueOn: server
!

changePointerControl: doAcceleration doThreshold: doThreshold
    accelerationNumerator: accelNumer accelerationDenominator:  accelDenom
    threshold: threshold
    | req |
    req _ XRequest fixedCommand: 105.
    req word: accelNumer; word: accelDenom; word: threshold; 
	bool: doAcceleration; bool: doThreshold.
    req queueOn: server
!

convertSelection: selection target: target property: property 
    requestor: requestor time: time
    | req |
    req _ XRequest fixedCommand: 24.
    req ulong: requestor id; ulong: selection id; ulong: target id;
	ulong: (XProtocol maybeMapId: property into: #(None));
    	ulong: (XProtocol maybeMap: time into: #(CurrentTime)).
    req queueOn: server
!

forceScreenSaver: mode
    | req |
    req _ XRequest fixedCommand: 115
		   aux: (XProtocol map: mode into: #(Reset Activate)).
    req queueOn: server
!

getFontPath
    | req numStrs strs |
    req _ XRequest fixedCommand: 52.
    req queueOn: server.

    server getReply.
    numStrs _ server uword.
    server skip: 22.
    strs _ Array new: numStrs.
    1 to: strs size do: [ :i | strs at: i put: server str ].
    server skipPad.
    ^GetFontPathReply path: strs
!

getInputFocus
    | req  revertTo focus |
    req _ XRequest fixedCommand: 43.
    req queueOn: server.

    revertTo _ server getMappedReply: #(None PointerRoot Parent).
    focus _ Window maybeIdFrom: server symbols: #(None PointerRoot).
    server skip: 20.
    ^GetInputFocusReply revertTo: revertTo focus: focus
!

getKeyboardControl
    | req keyClickPercent bellPercent bellPitch bellDuration ledMask
      globalAutoRepeat autoRepeats |
    req _ XRequest fixedCommand: 103.
    req queueOn: server.

    globalAutoRepeat _ server getMappedReply: #(Off On).
    ledMask _ server ulong.
    keyClickPercent _ server ubyte.
    bellPercent _ server ubyte.
    bellPitch _ server uword.
    bellDuration _ server uword.
    server skip: 2.
    autoRepeats _ server getBytes: 32.
    ^GetKeyboardControlReply globalAutoRepeat: globalAutoRepeat 
			     ledMask: ledMask keyClickPercent: keyClickPercent
			     bellPercent: bellPercent bellPitch: bellPitch
			     bellDuration: bellDuration 
			     autoRepeats: autoRepeats
!

getKeyboardMapping: firstKeycode count: count
    | req keysymsPerKeycode keysyms |
    req _ XRequest fixedCommand: 101.
    req ubyte: firstKeycode; ubyte: count; word: 0.
    req queueOn: server.

    keysymsPerKeycode _ server getReply.
    keysyms _ Array new: count * keysymsPerKeycode.
    server skip: 24.
    1 to: keysyms size do:
	[ :i | keysyms at: i put: (server ulong) ].
    ^GetKeyboardMappingReply keysymsPerKeycode: keysymsPerKeycode
			     keysyms: keysyms
!

getModifierMapping
    | req keycodesPerModifier keycodes |
    req _ XRequest fixecCommand: 119.
    req queueOn: server.

    keycodesPerModifier _ server getReply.
    server skip: 24.
    keycodes _ server getBytes: 8 * keycodesPerModifier.
    ^GetModifierMappingReply keycodesPerModifier: keycodesPerModifier
			     keycodes: keycodes
!

getPointerControl
    | req accelNumer accelDenom threshold |
    req _ XRequest fixedCommand: 106.
    req queueOn: server.

    server getReply.
    accelNumer _ server uword.    
    accelDenom _ server uword.    
    threshold _ server uword.    
    server skip: 18.
    ^GetPointerControlReply accelerationNumerator: accelNumer
			    accelerationDenominator: accelDenom
			    threshold: threshold
!

getPointerMapping
    | req mapLen map |
    req _ XRequest fixedCommand: 117.
    req queueOn: server.

    mapLen _ server getReply.
    server skip: 24.
    map _ server getBytes: mapLen.
    server skipPad.
    ^GetPointerMappingReply map: map
!

getScreenSaver
    | req timeout interval preferBlanking allowExposures |
    req _ XRequest fixedCommand: 108.
    req queueOn: server.

    server getReply.
    timeout _ server uword.
    interval _ server uword.
    preferBlanking _ XProtocol unmap: (server byte) into: #(No Yes).
    allowExposures _ XProtocol unmap: (server byte) into: #(No Yes).
    server skip: 18.
    ^GetScreenSaverReply timeout: timeout interval: interval 
			 preferBlanking: preferBlanking 
			 allowExposures: allowExposures
!

getSelectionOwner: selection
    | req owner |
    req _ XRequest fixedCommand: 23.
    req ulong: selection id.
    req queueOn: server.

    server getReply.
    owner _ Window maybeIdFrom: server symbols: #(None).
    server skip: 20.
    ^GetSelectionOwnerReply owner: owner
!

grabButton: modifiers button: button grabWindow: grabWindow 
    ownerEvents: ownerEvents eventMask: eventMask pointerMode: pointerMode
    keyboardMode: keyboardMode confineTo: confineTo cursor: cursor
    | req |
    req _ XRequest fixedCommand: 28
		   aux: (XProtocol mapBool: ownerEvents).
    req ulong: grabWindow id; 
	uword: (XProtocol mask: eventMask using: PointerEventNames);
	byte: (XProtocol map: pointerMode into: #(Synchronous Asynchronous));
	byte: (XProtocol map: keyboardMode into: #(Synchronous Asynchronous));
	ulong: (XProtocol maybeMapId: confineTo into: #(None));
	ulong: (XProtocol maybeMapId: cursor into: #(None));
	byte: (XProtocol map: button into: #(AnyButton));
	byte: 0;
	uword: (XProtocol mask: modifiers using: KeyMaskNames
			  and: #((AnyModifier 16r8000))).
    req queueOn: server.
!


grabKey: key modifiers: modifiers grabWindow: grabWindow 
    ownerEvents: ownerEvents pointerMode: pointerMode 
    keyboardMode: keyboardMode
    | req |
    req _ XRequest fixedCommand: 33
		   aux: (XProtocol mapBool: ownerEvents).
    req ulong: grabWindow id; 
	uword: (XProtocol mask: modifiers using: KeyMaskNames
			  and: #((AnyModifier 16r8000)));
	ubyte: (XProtocol maybeMap: key into: #(AnyKey));
	byte: (XProtocol map: pointerMode into: #(Synchronous Asynchronous));
	byte: (XProtocol map: keyboardMode into: #(Synchronous Asynchronous)).
    req queueOn: server.
!

grabKeyboard: grabWindow ownerEvents: ownerEvents pointerMode: pointerMode
    keyboardMode: keyboardMode time: time
    | req status |
    req _ XRequest fixedCommand: 31
		   aux: (XProtocol mapBool: ownerEvents).
    req ulong: grabWindow id; 
	ulong: (XProtocol maybeMap: time into: #(CurrentTime));
	byte: (XProtocol map: pointerMode into: #(Synchronous Asynchronous));
	byte: (XProtocol map: keyboardMode into: #(Synchronous Asynchronous)).
    req queueOn: server.
    
    status _ server getMappedReply: #(Success AlreadyGrabbed InvalidTime
					  NotViewable Frozen).
    server skip: 24.
    ^GrabKeyboardReply status: status
!

grabPointer: grabWindow ownerEvents: ownerEvents eventMask: eventMask
    pointerMode: pointerMode keyboardMode: keyboardMode confineTo: confineTo
    cursor: cursor time: time
    | req status |
    req _ XRequest fixedCommand: 26
		   aux: (XProtocol mapBool: ownerEvents).
    req ulong: grabWindow id; 
	uword: (XProtocol mask: eventMask using: PointerEventNames);
	byte: (XProtocol map: pointerMode into: #(Synchronous Asynchronous));
	byte: (XProtocol map: keyboardMode into: #(Synchronous Asynchronous));
	ulong: (XProtocol maybeMapId: confineTo into: #(None));
	ulong: (XProtocol maybeMapId: cursor into: #(None));
	ulong: (XProtocol maybeMap: time into: #(CurrentTime)).
    req queueOn: server.
    
    status _ server getMappedReply: #(Success AlreadyGrabbed InvalidTime
					  NotViewable Frozen).
    server skip: 24.
    ^GrabPointerReply status: status
!

grabServer
    | req |
    req _ XRequest fixedCommand: 36.
    req queueOn: server
!

killClient: resource
    | req |
    req _ XRequest fixedCommand: 113.
    req ulong: (XProtocol maybeMapId: resource into: #(AllTemporary)).
    req queueOn: server
!

listExtensions
    | req numStrs strs |
    req _ XRequest fixedCommand: 99.
    req queueOn: server.

    numStrs _ server getReply.
    server skip: 24.
    strs _ Array new: numStrs.
    1 to: numStrs do: [ :i | strs at: i put: server str ].
    server skipPad.
    ^ListExtensionsReply names: strs
!

listFonts: pattern maxNames: maxNames
    | req numStrs strs |
    req _ XRequest command: 49.
    req uword: maxNames; uword: pattern size; string: pattern.
    req queueOn: server.

    server getReply.
    numStrs _ server uword.
    server skip: 22.
    strs _ Array new: numStrs.
    1 to: numStrs do: [ :i | strs at: i put: server str ].
    server skipPad.
    ^ListFontsReply names: strs
!

listFontsWithInfo: pattern maxNames: maxNames
    | req replies nameLen minBounds maxBounds minCharOrByte2 maxCharOrByte2
      defaultChar numFontProps drawDirection minByte1 maxByte1 allCharsExist
      fontAscent fontDescent repliesHint properties name |
    req _ XRequest command: 50.
    req uword: maxNames; uword: pattern size; string: pattern.
    req queueOn: server.

    "server getReply."
    replies _ WriteStream on: (Array new: 1).
    [ nameLen _ server getReply.
      nameLen > 0 ] whileTrue: 
	  [ minBounds _ server charInfo.
	    server ulong.
	    maxBounds _ server charInfo.
	    server ulong.
	    minCharOrByte2 _ server uword.
	    maxCharOrByte2 _ server uword.
	    defaultChar _ server uword.
	    numFontProps _ server uword.
	    drawDirection _ XProtocol unmap: (server byte) 
				      into: #(LeftToRight RightToLeft).
	    minByte1 _ server ubyte.
	    maxByte1 _ server ubyte.
	    allCharsExist _ server bool.
	    fontAscent _ server word.
	    fontDescent _ server word.
	    repliesHint _ server ulong.
	    properties _ Array new: numFontProps.
	    1 to: numFontProps do:
		[ :i | properties at: i put: (server fontProp) ].
	    name _ server getString: nameLen.
	    server skipPad.
	    replies nextPut:
		(ListFontsWithInfoReply
		     minBounds: minBounds maxBounds: maxBounds 
		     minCharOrByte2: minCharOrByte2 
		     maxCharOrByte2: maxCharOrByte2 defaultChar: defaultChar
		     drawDirection: drawDirection minByte1: minByte1 
		     maxByte1: maxByte1 allCharsExist: allCharsExist 
		     fontAscent: fontAscent fontDescent: fontDescent
		     properties: properties name: name) ].
    server skip: 52.		"on the last one, there are 52 unused bytes"
    ^replies contents
!

listHosts
    | req numHosts hosts mode |
    req _ XRequest fixedCommand: 110.
    req queueOn: server.

    mode _ server getMappedReply: #(Disabled Enabled).
    numHosts _ server uword.
    server skip: 22.
    hosts _ Array new: numHosts.
    1 to: numHosts do: [ :i | hosts at: i put: server host ].
    ^ListHostsReply hosts: hosts
!

noOperation
    | req |
    req _ XRequest fixedCommand: 127.
    req queueOn: server
!

queryExtension: name
    | req present majorOpcode firstEvent firstError |
    req _ XRequest command: 98.
    req uword: (name size); uword: 0; string: name.
    req queueOn: server.

    server getReply.
    present _ server bool.
    majorOpcode _ server ubyte.
    firstEvent _ server ubyte.
    firstError _ server ubyte.
    server skip: 20.
    ^QueryExtensionReply present: present majorOpcode: majorOpcode
			 firstEvent: firstEvent firstError: firstError
!

queryKeymap
    | req keys |
    req _ XRequest fixedCommand: 44.
    req queueOn: server.

    server getReply.
    keys _ server getBytes: 32.
    ^QueryKeymapReply keys: keys
!

sendEvent: destination propagate: propagate eventMask: eventMask event: event
    | req keys |
    req _ XRequest fixedCommand: 25 
		   aux: (XProtocol mapBool: propagate).
    req ulong: (XProtocol maybeMapId: destination 
			  into: #(PointerWindow InputFocus));
	ulong: (XProtocol mask: eventMask using: EventNames).
    event eventTo: req.
    req queueOn: server
!

setAccessControl: mode
    | req keys |
    req _ XRequest fixedCommand: 111 
		   aux: (XProtocol map: mode into: #(Disable Enable)).
    req queueOn: server
!


setCloseDownMode: mode
    | req |
    req _ XRequest fixedCommand: 112 
		   aux: (XProtocol map: mode 
				   into: #(Destroy RetainPermanent
					       RetainTemporary)).
    req queueOn: server
!

setFontPath: path
    | req |
    req _ XRequest command: 51.
    req uword: path size; word: 0.
    path do: [ :str | req string: str ].
    req queueOn: server
!

setInputFocus: focus revertTo: revertTo time: time
    | req |
    req _ XRequest fixedCommand: 42
		   aux: (XProtocol map: revertTo
				   into: #(None PointerRoot Parent)).
    req ulong: (XProtocol maybeMapId: focus into: #(None PointerRoot));
	ulong: (XProtocol maybeMap: time into: #(CurrentTime)).
    req queueOn: server
!

setModifierMapping: keycodesPerModifier keycodes: keycodes
    | req status |
    req _ XRequest command: 118 aux: keycodesPerModifier.
    req bytes: keycodes.	"??? is this the right mapping?"
    req queueOn: server.
    
    status _ server getMappedReply: #(Success Busy Failed).
    server skip: 24.
    ^SetModifierMappingReply status: status
!

setPointerMapping: map
    | req status |
    req _ XRequest command: 116 aux: map size.
    req bytes: map.
    req queueOn: server.
    
    status _ server getMappedReply: #(Success Busy).
    server skip: 24.
    ^SetPointerMappingReply status: status
!

setScreenSaver: timeout interval: interval preferBlanking: preferBlanking
    allowExposures: allowExposures
    | req |
    req _ XRequest fixedCommand: 107.
    req word: timeout; word: interval; 
	byte: (XProtocol map: preferBlanking into: #(No Yes Default));
	byte: (XProtocol map: allowExposures into: #(No Yes Default));
	word: 0.
    req queueOn: server
!
    
setSelectionOwner: selection owner: owner time: time
    | req |
    req _ XRequest fixedCommand: 22.
    req ulong: (XProtocol maybeMapId: owner into: #(None));
	ulong: selection id;
	ulong: (XProtocol maybeMapId: time into: #(CurrentTime)).
    req queueOn: server
!
    
ungrabButton: modifiers button: button grabWindow: grabWindow
    | req |
    req _ XRequest fixedCommand: 29 
		   aux: (XProtocol maybeMap: button into: #(AnyButton)).
    req ulong: grabWindow id;
	uword: (XProtocol mask: modifiers using: KeyMaskNames
			  and: #((AnyModifier 16r8000)));
	uword: 0.
    req queueOn: server
!
    
ungrabKey: key modifiers: modifiers grabWindow: grabWindow 
    | req |
    req _ XRequest fixedCommand: 34
		   aux: (XProtocol maybeMap: key into: #(AnyKey)).
    req ulong: grabWindow id; 
	uword: (XProtocol mask: modifiers using: KeyMaskNames
			  and: #((AnyModifier 16r8000)));
	word: 0.
    req queueOn: server.
!

ungrabKeyboard: time
    | req |
    req _ XRequest fixedCommand: 32.
    req ulong: (XProtocol maybeMap: time into: #(CurrentTime)).
    req queueOn: server
!

ungrabPointer: time
    | req |
    req _ XRequest fixedCommand: 27.
    req ulong: (XProtocol maybeMap: time into: #(CurrentTime)).
    req queueOn: server
!

ungrabServer
    | req |
    req _ XRequest fixedCommand: 37.
    req queueOn: server
!

warpPointer: srcWindow dstWindow: dstWindow srcX: srcX srcY: srcY
    srcWidth: srcWidth srcHeight: srcHeight dstX: dstX dstY: dstY
    | req |
    req _ XRequest fixedCommand: 41.
    req ulong: (XProtocol maybeMapId: srcWindow into: #(None));
	ulong: (XProtocol maybeMapId: dstWindow into: #(None));
	word: srcX; word: srcY; uword: srcWidth; uword: srcHeight;
	word: dstX; word: dstY.
    req queueOn: server
! !




!Server methodsFor: 'private'!

init: hostName display: anInteger
    | fd req authRec | 

    authRec _ XAuth bestMatchFor: hostName.

    fd _ (self connectToServer: hostName display: anInteger).
    fd < 0
	ifTrue: [ ^self error: 'Failed to connect to server' ].
    socket _ UnixStream on: fd.

    server _ self.
    pos _ 0.    
    objects _ Dictionary new.
    queue _ OrderedCollection new.

    req _ XRequest simpleRequest.

    req byte: (Bigendian ifTrue: [ $B asciiValue ] ifFalse: [ $l asciiValue ]).
    req byte: 0.
    
    req word: 11.		"major version"
    req word: 0.		"minor version"
    req word: authRec name size.
    req word: authRec data size.
    req word: 0.		"unused"
    req bytes: authRec name.
    req pad.
    req bytes: authRec data.
    req pad.

    self bytes: req simpleContents.
    
    self byte == 1		"succeeded?"
	ifTrue: [ self byte.	"unused here "
		  
		  root _ self rootInit.
		  FastRWStream initialized.
		  root ]
	ifFalse: [ ^self error: 'Connection rejected' ]	
! !

Server initialize!

LoadStatus _ 'Server'!
