"======================================================================
|
|   Line style chooser for STIX paint program.
|
|   $Revision: 1.3 $
|   $Date: 1995/06/06 14:56:15 $
|   $Author: sbb $
|
 ======================================================================"


"======================================================================
|
| Copyright (C) 1994, 1995 Free Software Foundation, Inc.
| Written by Steve Byrne.
|
| This file is part of GNU Smalltalk.
|
| GNU Smalltalk is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 1, or (at your option) any later version.
| 
| GNU Smalltalk is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| GNU Smalltalk; see the file COPYING.  If not, write to the Free Software
| Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
|
 ======================================================================"

"
|     Change Log
| ============================================================================
| Author       Date       Change 
| sbb	     28 Aug 94	  Added header stuff.
|
"
ExclusiveChooser subclass: #StyleChooser
		 instanceVariableNames: 'gc styles'
		 classVariableNames: ''
		 poolDictionaries: ''
		 category: nil
!

!StyleChooser class methodsFor: 'instance creation'!

on: aWindow gc: aGc
    ^super new on: aWindow gc: aGc
! !



!StyleChooser methodsFor: 'operation'!

run
    window server processEvents.
!

stopRunning
    window server close.
    ExitHandlerLoop _ true.
! !


!StyleChooser methodsFor: 'assistance'!

drawStyles
    1 to: styles size do:
	[ :i | self drawStyle: i ].
    choice notNil
	ifTrue: [ self select: choice ].
	
! 

computePickAt: aPoint
    ^(((aPoint y // chunk) + 1) max: 1) min: styles size
!

select: aChoice
    | gc base blackPixel | 

    [
    blackPixel _ (window server root screens at: 1) blackPixel.

    gc _ GC createGC: window onServer: window server 
	    args: [ :r | r foreground: blackPixel;
			     lineWidth: 5 ].

    base _ (aChoice - 1) * chunk.

    window polyRectangle: gc
	   rectangles: (Array with: (0@(base+3) corner: width@(base+chunk - 2))).

    gc freeGC.
    ].
    self drawSelected: aChoice
! 

deselect: aChoice
    | gc base whitePixel | 

    [
    whitePixel _ (window server root screens at: 1) whitePixel.

    gc _ GC createGC: window onServer: window server 
	    args: [ :r | r foreground: whitePixel;
			     lineWidth: 5 ].

    base _ (aChoice - 1) * chunk.

    window polyRectangle: gc
	   rectangles: (Array with: (0@(base+3) corner: width@(base+chunk - 2))).

    gc freeGC.
    ].
    self drawDeselected: aChoice
!

drawStyle: aChoice
    | gc base blackPixel style | 

    blackPixel _ (window server root screens at: 1) blackPixel.

    style _ styles at: aChoice.

    gc _ GC createGC: window onServer: window server 
	    args: [ :r | r perform: (style at: 1) with: (style at: 2);
			     foreground: blackPixel ].

    base _ ((aChoice - 1) * chunk) + (chunk // 2).

    window polyLine: gc coordinateMode: #Origin
	   points: (Array with: 10@base 
			  with: (width - 10) @base).

    gc freeGC.
    self drawBorder: aChoice.
! !



!StyleChooser methodsFor: 'event handling'!

handleButtonPress: event
    | point style |
    point _ event eventX @ event eventY.
    self changeSelection: (self computePickAt: point).
    style _ styles at: choice.
    gc changeGC: [ :r | r perform: (style at: 1) with: (style at: 2) ]
! !


!StyleChooser methodsFor: 'private'!

on: aWindow gc: aGc
    | s screen cmap window |
    s _ aWindow server.
    screen _  s root screens at: 1.

    height _ 200.
    width _ 50.

    window _ Window createWindow: aWindow class: #CopyFromParent
		    depth: 0
		    visual: #CopyFromParent x: 50 y: 0
		    width: width height: height
		    borderWidth: 5 onServer: s
		    attrs: [ :r | r eventMask: #(Exposure ButtonPress );
				      backgroundPixel: screen whitePixel;
				      backingStore: #Always ].

    styles _ self makeStyles.
    chunk _ height // styles size.

    super on: window.

    gc _ aGc.

    self registerEventHandlers.    
    window mapWindow.
!

makeStyles
    ^#((lineWidth: 0)
	   (lineWidth: 1)
	   (lineWidth: 2)
	   (lineWidth: 3)
	   (lineWidth: 5)
	   (lineWidth: 8)
	   (lineWidth: 11)
	   (lineStyle: Solid)
	   (lineStyle: OnOffDash)
	   (lineStyle: DoubleDash)).
!

registerEventHandlers
    window registerFor: ExposeEvent
	   when: [ :e | e window == window id ]
	   do: [ :e | e count = 0 
			  ifTrue: [ self drawStyles ] ].
    window registerFor: ButtonPressEvent
	   when: [ :e | e event == window id ]
	   do: [ :e | self handleButtonPress: e ].
    
    " ### more "
! !

