"======================================================================
|
|   Utility classes for STIX painter program.
|
|   $Revision: 1.3 $
|   $Date: 1995/06/06 14:56:15 $
|   $Author: sbb $
|
 ======================================================================"


"======================================================================
|
| Copyright (C) 1994, 1995 Free Software Foundation, Inc.
| Written by Steve Byrne.
|
| This file is part of GNU Smalltalk.
|
| GNU Smalltalk is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 1, or (at your option) any later version.
| 
| GNU Smalltalk is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| GNU Smalltalk; see the file COPYING.  If not, write to the Free Software
| Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
|
 ======================================================================"

"
|     Change Log
| ============================================================================
| Author       Date       Change 
| sbb	     28 Aug 94	  Added header stuff.
|
"
Object subclass: #Chooser
       instanceVariableNames: 'window chunk width height liteGray darkGray 
                               stipple unselectedPixmap selectedPixmap'
       classVariableNames: ''
       poolDictionaries: ''
       category: nil
!

!Chooser methodsFor: 'generic rendering'!

drawSelected: aChoice
    self drawSelBorder: selectedPixmap at: aChoice.
    "self drawSelection: aChoice topColor: darkGray bottomColor: liteGray."
!

drawDeselected: aChoice
    self drawSelBorder: unselectedPixmap at: aChoice.
    "self drawSelection: aChoice topColor: liteGray bottomColor: darkGray."
!

drawSelBorder: pixmap at: aChoice
    | gc base |
    base _ (aChoice - 1) * chunk.

    GC withGC: window 
	args: [ :r | r clipMask: stipple;
			 clipYOrigin: base;
			 "clipYOrigin: base + 5;"
			 clipXOrigin: 0;
			 "clipXOrigin: 5;"
			 graphicsExposures: false ]
	do: [ :gc |     

		    window copyArea: pixmap
			   gc: gc
			   srcX: 0 srcY: 0
			   width: width height: chunk
			   dstX: 0 dstY: base.

		    ].
!




drawSelection: aChoice topColor: topColor bottomColor: bottomColor
    'in drawSelection' printNl.
    self drawSelectionAt: (aChoice - 1) * chunk on: window
	 topLeft: topColor bottomRight: bottomColor
!

drawSelectionAt: base on: drawable topLeft: topColor bottomRight: bottomColor
    | bottom top leftMargin rightMargin thick | 

    thick _ 3.

    leftMargin _ 2.
    rightMargin _ 1.
    bottom _ base + chunk - 1.
    top _ base + 2.

    GC withGC: drawable 
       args: [ :r | r foreground: topColor;
			lineWidth: thick ]
       do: [:gc | drawable polyLine: gc coordinateMode: #Origin
			   points: (Array with: leftMargin@(base + chunk)
					  with: leftMargin@top
					  with: width@top).
		  
		  gc changeGC: [ :r | r foreground: bottomColor ].

		  drawable fillPoly: gc
			   shape: #Convex
			   coordinateMode: #Previous
			   points: (Array with: (width)@base
					  with: 0@thick
					  with: (thick negated)@0).

		  drawable polyLine: gc coordinateMode: #Origin
			   points: (Array with: (width - rightMargin)@(base + thick)
					  with: (width - rightMargin)@bottom
					  with: thick@bottom).

		  drawable fillPoly: gc
			   shape: #Convex
			   coordinateMode: #Previous
			   points: (Array with: 0@(base+chunk)
					  with: thick@(thick negated)
					  with: 0@thick).
		  ].
! 


allocColor: colorName
    | cmap screen | 

    screen _ window server root screens at: 1.
    cmap _ screen defaultColormap.
    
    ^(cmap allocNamedColor: colorName) pixel
! !


!Chooser methodsFor: 'private'!

on: aWindow
    window _ aWindow.

    darkGray _ self allocColor: 'dim gray'.
    liteGray _ self allocColor: 'light gray'.

    self makeStipple.
    self makePixmap.
! 

makeStipple
    | gc |
    stipple _ Pixmap createPixmap: window depth: 1 
		     width: width height: chunk.
    
    gc _ GC createGC: stipple onServer: stipple server 
	    args: [ :r | r foreground: 0 ].
    stipple polyFillRectangle: gc
	    rectangles: (Array with: (0@0 corner: width @ chunk)).
    
    gc changeGC: [ :r | r foreground: 1; lineWidth: 3 ].

    stipple polyRectangle: gc
	    rectangles: (Array with: (2@2 corner: (width - 1) @ (chunk - 1))).

    gc freeGC.

!


makePixmap 
    | gc base blackPixel bottom top leftMargin rightMargin thick | 

    unselectedPixmap _ Pixmap createPixmap: window depth: 8 
			      width: width height: chunk.

    self drawSelectionAt: 0 on: unselectedPixmap
	 topLeft: darkGray bottomRight: liteGray.
    selectedPixmap _ Pixmap createPixmap: window depth: 8 
			    width: width height: chunk.

    self drawSelectionAt: 0 on: selectedPixmap
	 topLeft: liteGray bottomRight: darkGray.
! !





Chooser subclass: #ExclusiveChooser
	instanceVariableNames: '                            choice' 
	classVariableNames: ''
	poolDictionaries: ''
	category: nil
! 





"!ExclusiveChooser class methodsFor: 'instance creation'!


! !"


!ExclusiveChooser methodsFor: '###hacking'!

changeSelection: newChoice
    choice notNil
	ifTrue: [ self deselect: choice ].
    choice _ newChoice.
    self select: choice.
!

drawBorder: aChoice
    choice == aChoice 
	ifTrue: [self drawSelected: aChoice]
	ifFalse: [self drawDeselected: aChoice ].
!


select: aChoice
    self drawSelected: aChoice
!

deselect: aChoice
    self drawDeselected: aChoice
! !







ExclusiveChooser subclass: #ColorChooser
		 instanceVariableNames: 'gc colors'
		 classVariableNames: ''
		 poolDictionaries: ''
		 category: nil
!

!ColorChooser class methodsFor: 'instance creation'!

on: aWindow gc: aGc
    ^super new on: aWindow gc: aGc
! !



!ColorChooser methodsFor: 'operation'!

run
    window server processEvents.
!

stopRunning
    window server close.
    ExitHandlerLoop _ true.
! !


!ColorChooser methodsFor: 'assistance'!

drawColors
    1 to: colors size do:
	[ :i | self drawColor: i ].

    choice notNil
	ifTrue: [ self select: choice ].
	
! 

computePickAt: aPoint
    ^(((aPoint y // chunk) + 1) max: 1) min: colors size
!

select: aChoice
    super select: aChoice
!

deselect: aChoice
    "self drawColor: aChoice."
    super deselect: aChoice.
	
!

drawColor: aChoice
    | gc base blackPixel | 

    gc _ GC createGC: window onServer: window server 
	    args: [ :r | r foreground: (colors at: aChoice) ].

    base _ (aChoice - 1) * chunk.

    window polyFillRectangle: gc
	   rectangles: (Array with: (0@base corner: width@(base + chunk))).

    gc freeGC.

    self drawBorder: aChoice.
! !



!ColorChooser methodsFor: 'event handling'!

handleButtonPress: event
    | point pixel |
    point _ event eventX @ event eventY.
    self changeSelection: (self computePickAt: point).
    pixel _ colors at: choice.
    gc changeGC: [ :r | r foreground: pixel ]
! !





!ColorChooser methodsFor: 'private'!

on: aWindow gc: aGc
    | s screen cmap window colorNames |
    s _ aWindow server.
    screen _  s root screens at: 1.

    height _ 200.
    width _ 50 - 5.

    window _ Window createWindow: aWindow class: #CopyFromParent
		    depth: 0
		    visual: #CopyFromParent x: 0 y: 0
		    width: width height: height
		    borderWidth: 5 onServer: s
		    attrs: [ :r | r eventMask: #(Exposure ButtonPress );
				      backgroundPixel: screen whitePixel;
				      backingStore: #Always
				      ].
    window name: 'Paint It!'.

    colorNames _ #('red' 'green' 'blue' 'yellow' 'black' 'white' 'royal blue'
		       'pale green' 'tan' 'pink').

    chunk _ height // colorNames size.
    
    super on: window.

    colors _ self makeColorsFor: colorNames.

    gc _ aGc.

    self registerEventHandlers.    
    window mapWindow.
!

makeColorsFor: colorNames
    | cmap screen colors | 

 "    screen _ window server root screens at: 1.
    cmap _ screen defaultColormap."


    colors _ Array new: colorNames size.

    1 to: colorNames size do:
	[ :i | colors at: i put: (self allocColor: (colorNames at: i)) ].
 "    1 to: colorNames size do: 
	[ :i | colors at: i
		      put: (cmap allocNamedColor: (colorNames at: i)) pixel.
	       ]."
    ^colors
!

registerEventHandlers
    window registerFor: ExposeEvent
	   when: [ :e | e window == window id ]
	   do: [ :e | e count = 0 
			  ifTrue: [ self drawColors ] ].
    "window registerFor: NoExposureEvent
	   when: [ :e | true ]
	   do: [ :e | ]."
    window registerFor: ButtonPressEvent
	   when: [ :e | e event == window id ]
	   do: [ :e | self handleButtonPress: e ].
    
    " ### more "
! !

