"======================================================================
|
|   Smalltalk tokenizer.
|
|   $Revision: 1.4 $
|   $Date: 1995/06/06 14:54:50 $
|   $Author: sbb $
|
 ======================================================================"


"======================================================================
|
| Copyright (C) 1994, 1995 Free Software Foundation, Inc.
| Written by Steve Byrne.
|
| This file is part of GNU Smalltalk.
|
| GNU Smalltalk is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 1, or (at your option) any later version.
| 
| GNU Smalltalk is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| GNU Smalltalk; see the file COPYING.  If not, write to the Free Software
| Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
|
 ======================================================================"


"
|     Change Log
| ============================================================================
| Author       Date       Change 
|
"


FileStream fileIn: 'Token.st'!
FileStream fileIn: 'PushBackStream.st'!

Object subclass: #Tok
       instanceVariableNames: 'stream pushedBack tokenBuffer'
       classVariableNames: 'LexMethods LexExtra'
       poolDictionaries: ''
       category: ''
!

!Tok class methodsFor: 'initialization'!

initialize
    LexMethods _ Array new: 128.
    LexExtra _ Array new: 128.
    LexMethods atAllPut: #ignoreTok:.
    LexMethods at: (Character tab asciiValue) put: #whiteTok:.
    LexMethods at: (Character nl asciiValue) put: #whiteTok:.
    LexMethods at: (Character newPage asciiValue) put: #whiteTok:.
    LexMethods at: (Character cr asciiValue) put: #whiteTok:.
    #(
      ($  $  whiteTok:)
	  ($! $! binOpTok:)
    	  ($" $" commentTok:)
          ($# $# oneCharTok: Sharp)
    	  ($$ $$ charLitTok:)
    	  ($% $& binOpTok:)
    	  ($' $' stringTok:)
    	  ($( $( oneCharTok: OpenParen)
    	  ($) $) oneCharTok: CloseParen)
    	  ($* $- binOpTok:)
    	  ($. $. oneCharTok: Dot)
    	  ($/ $/ binOpTok:)
    	  ($0 $9 numberTok:)
	  ($: $: colonTok:)
    	  ($; $; oneCharTok: Semi)
	  ($< $@ binOpTok:)
	  ($A $Z idTok:)
    	  ($[ $[ oneCharTok: OpenBracket)
	  ($\ $\ binOpTok:)
    	  ($] $] oneCharTok: CloseBracket)
    	  ($^ $^ oneCharTok: UpArrow)
    	  ($_ $_ oneCharTok: Assign)
	  ($a $z idTok:)
	  ($| $| binOpTok:)
	  ($~ $~ binOpTok:)
	  )
	do: [ :range | self initRange: range ].
!

initRange: aRange
    | method |
    method _ aRange at: 3.
    (aRange at: 1) asciiValue to: (aRange at: 2) asciiValue do:
	[ :ch | LexMethods at: ch put: method.
    	    	aRange size = 4 ifTrue: 
		    [ LexExtra at: ch put: (aRange at: 4) ]
		]
! !


!Tok class methodsFor: 'instance creation'!

on: fileName
    ^self new init: fileName
! !

!Tok methodsFor: 'hacking'!

next
    | ch tok |
    tokenBuffer notNil
	ifTrue: [ tok _ tokenBuffer.
		  tokenBuffer _ nil.
		  ^tok ].
	
    [ stream atEnd ifTrue: [ ^nil ].
      ch _ stream next.
      tok _ self perform: (LexMethods at: ch asciiValue) with: ch.
      tok isNil ] whileTrue.
    ^tok
!

peek
    tokenBuffer isNil
	ifTrue: [ tokenBuffer _ self next ].
    ^tokenBuffer
!

atEnd
    ^stream atEnd
!

close
    stream close
!!

!Tok methodsFor: 'token parsing'!

whiteTok: aChar
    "Gobble the char and return nothing"
    ^nil
!

colonTok: aChar
    | ch |
    ch _ stream peek.
    ch == $=
	ifTrue: [ stream next.
		  ^AssignTok new ]
	ifFalse: [ ^ColonTok new ]
!

charLitTok: aChar
    "Called with aChar == $$"
    | ch |
    ^LiteralTok value: stream next
!			     

idTok: aChar
    stream putBack: aChar.
    ^self parseIdent
!

commentTok: aChar
    | bs ch |
    bs _ WriteStream on: (String new: 10).
    [ ch _ stream next.
      ch == $" 
	  ifTrue: [ stream peek == $" 
			ifTrue: [ stream next. true ]
			ifFalse: [ false ] ]
	  ifFalse: [ true ] ]
	whileTrue: [ bs nextPut: ch ].
    "^CommentTok value: bs contents"
    ^nil "ignore comments for now "
!

stringTok: aChar
    | bs ch |    
    bs _ WriteStream on: (String new: 10).
    [ ch _ stream next.
      ch == $' 
	  ifTrue: [ stream peek == $' 
			ifTrue: [ stream next. true ]
			ifFalse: [ false ] ]
	  ifFalse: [ true ] ]
	whileTrue: [ bs nextPut: ch ].
    ^LiteralTok value: bs contents 
!

oneCharTok: aChar
    ^(Smalltalk at: ((LexExtra at: (aChar asciiValue)), 'Tok') asSymbol)
	new
!

binOpTok: aChar
    | bs ch cont |
    bs _ WriteStream on: (String new: 2).
    ch _ stream peek.
    aChar == $< ifTrue:
	[ ch == $p ifTrue: [ (self primitiveTok: ch)
				 ifTrue: [ ^PrimitiveStartTok new ] ] ].
    bs nextPut: aChar.

    (self isSpecial: ch)
	ifTrue: [ bs nextPut: stream next ]
	ifFalse: [ ((aChar == $-) and: [ ch isDigit ])
		       ifTrue: [ ^self numberTok: aChar ] ].
    cont _ bs contents.
    cont = '!' ifTrue: [ ^BangTok new ].
    cont = '|' ifTrue: [ ^VerticalBarTok new ].
    ^BinopTok value: cont
!

numberTok: aChar
    | mantissaParsed isNegative dotSeen base exponent scale ch num 
	floatExponent |
    mantissaParsed _ isNegative _ dotSeen _ false.
    
    base _ 10.
    exponent _ 0.
    scale _ 0.0.


    ch _ aChar.    
    stream putBack: ch.		"written with peeking in mind"
    ch == $-	ifFalse: 
	[ "could be radix"
	  num _ self parseDigits: ch base: 10.
	  ch _ stream peek.
	  ch == $r
	      ifTrue: [ base _ num truncated.
			stream next. "skip over 'r'"
			ch _ stream peek ]
	      ifFalse: [ mantissaParsed _ true ] ].
    
" here we've either
  a) parsed base, an 'r' and are sitting on the following character
  b) parsed the integer part of the mantissa, and are sitting on the char
     following it, or
  c) parsed nothing and are sitting on a - sign.
"

    ch == $- 
	ifTrue:	
	    [ mantissaParsed
		  ifTrue: [ ^self parseError: 'malformed number' ].
	      isNegative _ true.
	      stream next.		"skip '-'"
	      ch _ stream peek ].
    (self isDigit: ch base: base) ifTrue: 
	[ num _ self parseDigits: ch base: base.
	  mantissaParsed _ true ].
    
    ch _ stream peek.

    ch == $.
    	ifTrue: [ "###check mantissa parsed here"
		  stream next.
		  ch _ stream peek.
		  ch isDigit 
		      ifTrue:
			  [ dotSeen _ true.
			    self parseFraction: ch 
				 num: num
				 base: base
				 return: [ :n :s | num _ n. scale _ s ]. ]
		      ifFalse: 
			  [ stream putBack: $..
			    ch _ $. ]
		      ].
    ch == $e
	ifTrue: 
	    [ stream next.	"skip over 'e'"
	      ch _ stream peek.
	      ch == $-
		  ifTrue: 
		      [ stream next.
			floatExponent _ self parseDigits: stream peek
					     base: 10.
			exponent _ exponent - floatExponent ]
		  ifFalse: 
		      [ floatExponent _ self parseDigits: ch
					     base: 10.
			exponent _ exponent + floatExponent ]
		  ].
    
    scale ~= 0.0
	ifTrue: [ num _ num * scale ].
    exponent ~= 0
	ifTrue: [ num _ num raisedToInteger: exponent ].

    isNegative
	ifTrue: [ num _ num negated ].

    dotSeen 
	ifTrue: [ ^LiteralTok value: num ]
	ifFalse: 
	    [ (num between: -1073741824.0 and: 1073741823.0)
		  ifFalse: [ ^self parseError: 'Integer literal too large' ].
	      ^LiteralTok value: num truncated ]
! !

!Tok methodsFor: 'utility methods'!

isSpecial: ch
    "SBB -- removed bang 25-Apr-93 02:09:55 ; was causing bin op lexer
     to get greedy with !!"
    ^'%&*+,-/<=>?@\|~' includes: ch
! 

parseDigits: ch base: base
    | c num |
    c _ ch.
    num _ 0.0.			"accumulate FP in case we're really getting FP"
    [ self isDigit: c base: base ] whileTrue:
	[ num _ num * base + c digitValue.
	  stream next. 
	  c _ stream peek ].
    ^num
!


parseFraction: ch num: num base: base return: aBlock
    | c scale |
    c _ ch.
    scale _ 1.0.
    [ self isDigit: c base: base ] whileTrue:
	[ num _ num * base + c digitValue.
	  stream next. 
	  c _ stream peek.
	  scale _ scale / base.
	  ].
    aBlock value: num value: scale

!

isDigit: aChar base: base
    ^((aChar between: $0 and: $9)
	| (aChar between: $A and: $Z) )
	and: [ aChar digitValue < base ]
!

resetRecording
    stream resetRecording
!

recording
    ^stream recording
!

close				"probably a better name exists for this"
    stream close

! !


!Tok methodsFor: 'private'!

init: aFileName
    stream _ PushBackStream on: (FileStream open: aFileName mode: 'r')
!

parseIdent
    | s colonsSeen ch lastChar id |
    s _ WriteStream on: (String new: 1).
    colonsSeen _ 0.
    lastChar _ nil.
    [ ch _ stream peek.
      ch isLetter or: [ ch isDigit or: [ (ch == $:) and: [ (lastChar ~~ ch) ] ] ] ]
	whileTrue: [ s nextPut: ch.
		     lastChar _ ch.
		     (ch == $:)
			 ifTrue: [ colonsSeen _ colonsSeen + 1 ].
		     stream next ].
    id _ s contents.
    colonsSeen > 0
	ifTrue: 
	    [ lastChar == $:
		  ifFalse: [ ^self parseError: 'Bad keyword identifier' ].
	      colonsSeen == 1
		  ifTrue: [ ^KeywordTok value: id ]
		  ifFalse: [ ^SymbolKeywordTok value: id ] ]
	ifFalse:
	    [ ^IdentifierTok value: id ].
! 

primitiveTok: aChar
    | id |
    id _ self idTok: stream next.
    ^id value = 'primitive:'
! !

Tok initialize!

"

| f tok |
    f _ Tok on: 'ntt.st'.
    ""f _ Tok on: 'builtins.st'.""
    [ tok _ f next.
      tok notNil ] whileTrue"": [ tok inspect ]""
!

| fileNameStream fileNames f tok |
    fileNameStream _ FileStream popen: 'ls stix/[A-Z]*.st' dir: 'r'.
    fileNames _ TokenStream onStream: fileNameStream.
    fileNames do: 
	[ :name | stdout nextPutAll: 'Processing ';
		      nextPutAll: name; nl.
		  f _ Tok on: name.
		  [ tok _ f next.
			     tok notNil ] whileTrue.
		  f close ]
!

"
